@@ -14,10 +14,6 @@
 
 #include "safeguards.h"
 
-BaseConsist::~BaseConsist()
-{
-	free(this->name);
-}
 
 /**
  * Copy properties of other BaseConsist.
@@ -27,8 +23,7 @@ void BaseConsist::CopyConsistPropertiesFrom(const BaseConsist *src)
 {
 	if (this == src) return;
 
-	free(this->name);
-	this->name = src->name != nullptr ? stredup(src->name) : nullptr;
+	this->name = src->name;
 
 	this->current_order_time = src->current_order_time;
 	this->lateness_counter = src->lateness_counter;
