@@ -34,7 +34,7 @@ static inline void InsertSecondUint32(const uint32 value, __m128i &into)
 
 static inline void LoadUint64(const uint64 value, __m128i &into)
 {
-#ifdef _SQ64
+#ifdef POINTER_IS_64BIT
 	into = _mm_cvtsi64_si128(value);
 #else
 	#if (SSE_VERSION >= 4)
@@ -297,7 +297,7 @@ inline void Blitter_32bppSSE4::Draw(const Blitter::BlitterParams *bp, ZoomLevel
 							m_colour = r == 0 ? m_colour : cmap; \
 							m_colour = m != 0 ? m_colour : srcm; \
 							}
-#ifdef _SQ64
+#ifdef POINTER_IS_64BIT
 						uint64 srcs = _mm_cvtsi128_si64(srcABCD);
 						uint64 remapped_src = 0;
 						CMOV_REMAP(c0, 0, srcs, mvX2);
