@@ -51,7 +51,7 @@ struct Backup {
 		{
 			/* We cannot assert here, as missing restoration is 'normal' when exceptions are thrown.
 			 * Exceptions are especially used to abort world generation. */
-			DEBUG(misc, 0, "%s:%d: Backed-up value was not restored!", this->file, this->line);
+			Debug(misc, 0, "{}:{}: Backed-up value was not restored!", this->file, this->line);
 			this->Restore();
 		}
 	}
