@@ -12,8 +12,8 @@
 
 #if defined(__APPLE__)
 	/* Mac OS X already has both Rect and Point declared */
-	#define Rect OTTD_Rect
-	#define Point OTTD_Point
+#	define Rect OTTD_Rect
+#	define Point OTTD_Point
 #endif /* __APPLE__ */
 
 
@@ -62,10 +62,4 @@ struct PointDimension {
 	int height;
 };
 
-/** A pair of two integers */
-struct Pair {
-	int a;
-	int b;
-};
-
 #endif /* GEOMETRY_TYPE_HPP */
