@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -21,8 +19,8 @@
 /* virtual */ PoolBase::~PoolBase()
 {
 	PoolVector *pools = PoolBase::GetPools();
-	pools->Erase(pools->Find(this));
-	if (pools->Length() == 0) delete pools;
+	pools->erase(std::find(pools->begin(), pools->end(), this));
+	if (pools->size() == 0) delete pools;
 }
 
 /**
@@ -31,10 +29,7 @@
  */
 /* static */ void PoolBase::Clean(PoolType pt)
 {
-	PoolVector *pools = PoolBase::GetPools();
-	PoolBase **end = pools->End();
-	for (PoolBase **ppool = pools->Begin(); ppool != end; ppool++) {
-		PoolBase *pool = *ppool;
+	for (PoolBase *pool : *PoolBase::GetPools()) {
 		if (pool->type & pt) pool->CleanPool();
 	}
 }
