@@ -28,6 +28,9 @@ protected:
 	int ascender;                     ///< The ascender value of the font.
 	int descender;                    ///< The descender value of the font.
 	int units_per_em;                 ///< The units per EM value of the font.
+
+	static int GetDefaultFontHeight(FontSize fs);
+
 public:
 	FontCache(FontSize fs);
 	virtual ~FontCache();
@@ -42,7 +45,7 @@ public:
 	 * Get the height of the font.
 	 * @return The height of the font.
 	 */
-	virtual int GetHeight() const { return this->height; }
+	inline int GetHeight() const { return this->height; }
 
 	/**
 	 * Get the ascender value of the font.
@@ -127,7 +130,7 @@ public:
 	 * Get the native OS font handle, if there is one.
 	 * @return Opaque OS font handle.
 	 */
-	virtual void *GetOSHandle()
+	virtual const void *GetOSHandle()
 	{
 		return nullptr;
 	}
@@ -209,15 +212,13 @@ static inline bool GetDrawGlyphShadow(FontSize size)
 	return FontCache::Get(size)->GetDrawGlyphShadow();
 }
 
-#if defined(WITH_FREETYPE) || defined(_WIN32)
-
 /** Settings for a single freetype font. */
 struct FreeTypeSubSetting {
-	char font[MAX_PATH]; ///< The name of the font, or path to the font.
-	uint size;           ///< The (requested) size of the font.
-	bool aa;             ///< Whether to do anti aliasing or not.
+	std::string font; ///< The name of the font, or path to the font.
+	uint size;        ///< The (requested) size of the font.
+	bool aa;          ///< Whether to do anti aliasing or not.
 
-	const void *os_handle = nullptr; ///< Optional native OS font info.
+	const void *os_handle = nullptr; ///< Optional native OS font info. Only valid during font search.
 };
 
 /** Settings for the freetype fonts. */
@@ -230,9 +231,8 @@ struct FreeTypeSettings {
 
 extern FreeTypeSettings _freetype;
 
-#endif /* defined(WITH_FREETYPE) || defined(_WIN32) */
-
 void InitFreeType(bool monospace);
 void UninitFreeType();
+bool HasAntialiasedFonts();
 
 #endif /* FONTCACHE_H */
