@@ -9,8 +9,10 @@
 
 #include "stdafx.h"
 #include <math.h>
+#include <mutex>
 #include "core/math_func.hpp"
 #include "framerate_type.h"
+#include "settings_type.h"
 
 #include "safeguards.h"
 #include "mixer.h"
@@ -38,6 +40,7 @@ static MixerChannel _channels[8];
 static uint32 _play_rate = 11025;
 static uint32 _max_size = UINT_MAX;
 static MxStreamCallback _music_stream = nullptr;
+static std::mutex _music_stream_mutex;
 
 /**
  * The theoretical maximum volume for a single sound sample. Multiple sound
@@ -45,7 +48,7 @@ static MxStreamCallback _music_stream = nullptr;
  * stops overflowing when too many sounds are played at the same time, which
  * causes an even worse sound quality.
  */
-static const int MAX_VOLUME = 128 * 128;
+static const int MAX_VOLUME = 32767;
 
 /**
  * Perform the rate conversion between the input and output.
@@ -60,7 +63,7 @@ static int RateConversion(T *b, int frac_pos)
 	return ((b[0] * ((1 << 16) - frac_pos)) + (b[1] * frac_pos)) >> 16;
 }
 
-static void mix_int16(MixerChannel *sc, int16 *buffer, uint samples)
+static void mix_int16(MixerChannel *sc, int16 *buffer, uint samples, uint8 effect_vol)
 {
 	if (samples > sc->samples_left) samples = sc->samples_left;
 	sc->samples_left -= samples;
@@ -69,8 +72,8 @@ static void mix_int16(MixerChannel *sc, int16 *buffer, uint samples)
 	const int16 *b = (const int16 *)sc->memory + sc->pos;
 	uint32 frac_pos = sc->frac_pos;
 	uint32 frac_speed = sc->frac_speed;
-	int volume_left = sc->volume_left;
-	int volume_right = sc->volume_right;
+	int volume_left = sc->volume_left * effect_vol / 255;
+	int volume_right = sc->volume_right * effect_vol / 255;
 
 	if (frac_speed == 0x10000) {
 		/* Special case when frac_speed is 0x10000 */
@@ -96,7 +99,7 @@ static void mix_int16(MixerChannel *sc, int16 *buffer, uint samples)
 	sc->pos = b - (const int16 *)sc->memory;
 }
 
-static void mix_int8_to_int16(MixerChannel *sc, int16 *buffer, uint samples)
+static void mix_int8_to_int16(MixerChannel *sc, int16 *buffer, uint samples, uint8 effect_vol)
 {
 	if (samples > sc->samples_left) samples = sc->samples_left;
 	sc->samples_left -= samples;
@@ -105,8 +108,8 @@ static void mix_int8_to_int16(MixerChannel *sc, int16 *buffer, uint samples)
 	const int8 *b = sc->memory + sc->pos;
 	uint32 frac_pos = sc->frac_pos;
 	uint32 frac_speed = sc->frac_speed;
-	int volume_left = sc->volume_left;
-	int volume_right = sc->volume_right;
+	int volume_left = sc->volume_left * effect_vol / 255;
+	int volume_right = sc->volume_right * effect_vol / 255;
 
 	if (frac_speed == 0x10000) {
 		/* Special case when frac_speed is 0x10000 */
@@ -151,16 +154,27 @@ void MxMixSamples(void *buffer, uint samples)
 	/* Clear the buffer */
 	memset(buffer, 0, sizeof(int16) * 2 * samples);
 
-	/* Fetch music if a sampled stream is available */
-	if (_music_stream) _music_stream((int16*)buffer, samples);
+	{
+		std::lock_guard<std::mutex> lock{ _music_stream_mutex };
+		/* Fetch music if a sampled stream is available */
+		if (_music_stream) _music_stream((int16*)buffer, samples);
+	}
+
+	/* Apply simple x^3 scaling to master effect volume. This increases the
+	 * perceived difference in loudness to better match expectations. effect_vol
+	 * is expected to be in the range 0-127 hence the division by 127 * 127 to
+	 * get back into range. */
+	uint8 effect_vol = (_settings_client.music.effect_vol *
+	                    _settings_client.music.effect_vol *
+	                    _settings_client.music.effect_vol) / (127 * 127);
 
 	/* Mix each channel */
 	for (mc = _channels; mc != endof(_channels); mc++) {
 		if (mc->active) {
 			if (mc->is16bit) {
-				mix_int16(mc, (int16*)buffer, samples);
+				mix_int16(mc, (int16*)buffer, samples, effect_vol);
 			} else {
-				mix_int8_to_int16(mc, (int16*)buffer, samples);
+				mix_int8_to_int16(mc, (int16*)buffer, samples, effect_vol);
 			}
 			if (mc->samples_left == 0) MxCloseChannel(mc);
 		}
@@ -227,6 +241,7 @@ void MxActivateChannel(MixerChannel *mc)
  */
 uint32 MxSetMusicSource(MxStreamCallback music_callback)
 {
+	std::lock_guard<std::mutex> lock{ _music_stream_mutex };
 	_music_stream = music_callback;
 	return _play_rate;
 }
@@ -234,6 +249,7 @@ uint32 MxSetMusicSource(MxStreamCallback music_callback)
 
 bool MxInitialize(uint rate)
 {
+	std::lock_guard<std::mutex> lock{ _music_stream_mutex };
 	_play_rate = rate;
 	_max_size  = UINT_MAX / _play_rate;
 	_music_stream = nullptr; /* rate may have changed, any music source is now invalid */
