@@ -16,6 +16,7 @@
 #include "tcp.h"
 #include "../network_type.h"
 #include "../../core/pool_type.hpp"
+#include <chrono>
 
 /**
  * Enum with all types of TCP packets.
@@ -23,7 +24,7 @@
  */
 enum PacketGameType {
 	/*
-	 * These first three pair of packets (thus six in
+	 * These first four pair of packets (thus eight in
 	 * total) must remain in this order for backward
 	 * and forward compatibility between clients that
 	 * are trying to join directly.
@@ -37,9 +38,13 @@ enum PacketGameType {
 	PACKET_CLIENT_JOIN,                  ///< The client telling the server it wants to join.
 	PACKET_SERVER_ERROR,                 ///< Server sending an error message to the client.
 
-	/* Packets used for the pre-game lobby. */
-	PACKET_CLIENT_COMPANY_INFO,          ///< Request information about all companies.
-	PACKET_SERVER_COMPANY_INFO,          ///< Information about a single company.
+	/* Unused packet types, formerly used for the pre-game lobby. */
+	PACKET_CLIENT_UNUSED,                ///< Unused.
+	PACKET_SERVER_UNUSED,                ///< Unused.
+
+	/* Packets used to get the game info. */
+	PACKET_SERVER_GAME_INFO,             ///< Information about the server.
+	PACKET_CLIENT_GAME_INFO,             ///< Request information about the server.
 
 	/*
 	 * Packets after here assume that the client
@@ -94,6 +99,7 @@ enum PacketGameType {
 	/* Human communication! */
 	PACKET_CLIENT_CHAT,                  ///< Client said something that should be distributed.
 	PACKET_SERVER_CHAT,                  ///< Server distributing the message of a client (or itself).
+	PACKET_SERVER_EXTERNAL_CHAT,         ///< Server distributing the message from external source.
 
 	/* Remote console. */
 	PACKET_CLIENT_RCON,                  ///< Client asks the server to execute some command.
@@ -183,38 +189,17 @@ protected:
 	virtual NetworkRecvStatus Receive_SERVER_ERROR(Packet *p);
 
 	/**
-	 * Request company information (in detail).
+	 * Request game information.
 	 * @param p The packet that was just received.
 	 */
-	virtual NetworkRecvStatus Receive_CLIENT_COMPANY_INFO(Packet *p);
+	virtual NetworkRecvStatus Receive_CLIENT_GAME_INFO(Packet *p);
 
 	/**
-	 * Sends information about the companies (one packet per company):
-	 * uint8   Version of the structure of this packet (NETWORK_COMPANY_INFO_VERSION).
-	 * bool    Contains data (false marks the end of updates).
-	 * uint8   ID of the company.
-	 * string  Name of the company.
-	 * uint32  Year the company was inaugurated.
-	 * uint64  Value.
-	 * uint64  Money.
-	 * uint64  Income.
-	 * uint16  Performance (last quarter).
-	 * bool    Company is password protected.
-	 * uint16  Number of trains.
-	 * uint16  Number of lorries.
-	 * uint16  Number of busses.
-	 * uint16  Number of planes.
-	 * uint16  Number of ships.
-	 * uint16  Number of train stations.
-	 * uint16  Number of lorry stations.
-	 * uint16  Number of bus stops.
-	 * uint16  Number of airports and heliports.
-	 * uint16  Number of harbours.
-	 * bool    Company is an AI.
-	 * string  Client names (comma separated list)
+	 * Sends information about the game.
+	 * Serialized NetworkGameInfo. See game_info.h for details.
 	 * @param p The packet that was just received.
 	 */
-	virtual NetworkRecvStatus Receive_SERVER_COMPANY_INFO(Packet *p);
+	virtual NetworkRecvStatus Receive_SERVER_GAME_INFO(Packet *p);
 
 	/**
 	 * Send information about a client:
@@ -256,7 +241,7 @@ protected:
 	virtual NetworkRecvStatus Receive_CLIENT_COMPANY_PASSWORD(Packet *p);
 
 	/**
-	 * The client is joined and ready to receive his map:
+	 * The client is joined and ready to receive their map:
 	 * uint32  Own client ID.
 	 * uint32  Generation seed.
 	 * string  Network ID of the server.
@@ -394,6 +379,16 @@ protected:
 	 */
 	virtual NetworkRecvStatus Receive_SERVER_CHAT(Packet *p);
 
+	/**
+	 * Sends a chat-packet for external source to the client:
+	 * string  Name of the source this message came from.
+	 * uint16  TextColour to use for the message.
+	 * string  Name of the user who sent the messsage.
+	 * string  Message (max NETWORK_CHAT_LENGTH).
+	 * @param p The packet that was just received.
+	 */
+	virtual NetworkRecvStatus Receive_SERVER_EXTERNAL_CHAT(Packet *p);
+
 	/**
 	 * Set the password for the clients current company:
 	 * string  The password.
@@ -518,7 +513,7 @@ public:
 	uint32 last_frame;           ///< Last frame we have executed
 	uint32 last_frame_server;    ///< Last frame the server has executed
 	CommandQueue incoming_queue; ///< The command-queue awaiting handling
-	uint last_packet;            ///< Time we received the last frame.
+	std::chrono::steady_clock::time_point last_packet; ///< Time we received the last frame.
 
 	NetworkRecvStatus CloseConnection(bool error = true) override;
 
