@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -151,7 +149,7 @@ static uint32 GetObjectIDAtOffset(TileIndex tile, uint32 cur_grfid)
 	const ObjectSpec *spec = ObjectSpec::Get(o->type);
 
 	/* Default objects have no associated NewGRF file */
-	if (spec->grf_prop.grffile == NULL) {
+	if (spec->grf_prop.grffile == nullptr) {
 		return 0xFFFE; // Defined in another grf file
 	}
 
@@ -188,11 +186,10 @@ static uint32 GetNearbyObjectTileInformation(byte parameter, TileIndex tile, Obj
 static uint32 GetClosestObject(TileIndex tile, ObjectType type, const Object *current)
 {
 	uint32 best_dist = UINT32_MAX;
-	const Object *o;
-	FOR_ALL_OBJECTS(o) {
+	for (const Object *o : Object::Iterate()) {
 		if (o->type != type || o == current) continue;
 
-		best_dist = min(best_dist, DistanceManhattan(tile, o->location.tile));
+		best_dist = std::min(best_dist, DistanceManhattan(tile, o->location.tile));
 	}
 
 	return best_dist;
@@ -229,7 +226,7 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte local_id, uint32 grfid,
 	/* If the object type is invalid, there is none and the closest is far away. */
 	if (idx >= NUM_OBJECTS) return 0 | 0xFFFF;
 
-	return Object::GetTypeCount(idx) << 16 | min(GetClosestObject(tile, idx, current), 0xFFFF);
+	return Object::GetTypeCount(idx) << 16 | std::min(GetClosestObject(tile, idx, current), 0xFFFFu);
 }
 
 /** Used by the resolver to get values for feature 0F deterministic spritegroups. */
@@ -237,9 +234,9 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte local_id, uint32 grfid,
 {
 	/* We get the town from the object, or we calculate the closest
 	 * town if we need to when there's no object. */
-	const Town *t = NULL;
+	const Town *t = nullptr;
 
-	if (this->obj == NULL) {
+	if (this->obj == nullptr) {
 		switch (variable) {
 			/* Allow these when there's no object. */
 			case 0x41:
@@ -304,10 +301,10 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte local_id, uint32 grfid,
 		case 0x44: return GetTileOwner(this->tile);
 
 		/* Get town zone and Manhattan distance of closest town */
-		case 0x45: return GetTownRadiusGroup(t, this->tile) << 16 | min(DistanceManhattan(this->tile, t->xy), 0xFFFF);
+		case 0x45: return GetTownRadiusGroup(t, this->tile) << 16 | std::min(DistanceManhattan(this->tile, t->xy), 0xFFFFu);
 
-		/* Get square of Euclidian distance of closes town */
-		case 0x46: return GetTownRadiusGroup(t, this->tile) << 16 | min(DistanceSquare(this->tile, t->xy), 0xFFFF);
+		/* Get square of Euclidian distance of closest town */
+		case 0x46: return DistanceSquare(this->tile, t->xy);
 
 		/* Object colour */
 		case 0x47: return this->obj->colour;
@@ -325,7 +322,7 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte local_id, uint32 grfid,
 		}
 
 		/* Land info of nearby tiles */
-		case 0x62: return GetNearbyObjectTileInformation(parameter, this->tile, this->obj == NULL ? INVALID_OBJECT : this->obj->index, this->ro.grffile->grf_version >= 8);
+		case 0x62: return GetNearbyObjectTileInformation(parameter, this->tile, this->obj == nullptr ? INVALID_OBJECT : this->obj->index, this->ro.grffile->grf_version >= 8);
 
 		/* Animation counter of nearby tile */
 		case 0x63: {
@@ -338,7 +335,7 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte local_id, uint32 grfid,
 	}
 
 unhandled:
-	DEBUG(grf, 1, "Unhandled object variable 0x%X", variable);
+	Debug(grf, 1, "Unhandled object variable 0x{:X}", variable);
 
 	*available = false;
 	return UINT_MAX;
@@ -355,10 +352,10 @@ unhandled:
  */
 ObjectResolverObject::ObjectResolverObject(const ObjectSpec *spec, Object *obj, TileIndex tile, uint8 view,
 		CallbackID callback, uint32 param1, uint32 param2)
-	: ResolverObject(spec->grf_prop.grffile, callback, param1, param2), object_scope(*this, obj, tile, view)
+	: ResolverObject(spec->grf_prop.grffile, callback, param1, param2), object_scope(*this, obj, spec, tile, view)
 {
-	this->town_scope = NULL;
-	this->root_spritegroup = (obj == NULL && spec->grf_prop.spritegroup[CT_PURCHASE_OBJECT] != NULL) ?
+	this->town_scope = nullptr;
+	this->root_spritegroup = (obj == nullptr && spec->grf_prop.spritegroup[CT_PURCHASE_OBJECT] != nullptr) ?
 			spec->grf_prop.spritegroup[CT_PURCHASE_OBJECT] : spec->grf_prop.spritegroup[0];
 }
 
@@ -374,19 +371,29 @@ ObjectResolverObject::~ObjectResolverObject()
  */
 TownScopeResolver *ObjectResolverObject::GetTown()
 {
-	if (this->town_scope == NULL) {
+	if (this->town_scope == nullptr) {
 		Town *t;
-		if (this->object_scope.obj != NULL) {
+		if (this->object_scope.obj != nullptr) {
 			t = this->object_scope.obj->town;
 		} else {
 			t = ClosestTownFromTile(this->object_scope.tile, UINT_MAX);
 		}
-		if (t == NULL) return NULL;
-		this->town_scope = new TownScopeResolver(*this, t, this->object_scope.obj == NULL);
+		if (t == nullptr) return nullptr;
+		this->town_scope = new TownScopeResolver(*this, t, this->object_scope.obj == nullptr);
 	}
 	return this->town_scope;
 }
 
+GrfSpecFeature ObjectResolverObject::GetFeature() const
+{
+	return GSF_OBJECTS;
+}
+
+uint32 ObjectResolverObject::GetDebugID() const
+{
+	return this->object_scope.spec->grf_prop.local_id;
+}
+
 /**
  * Perform a callback for an object.
  * @param callback The callback to perform.
@@ -395,7 +402,7 @@ TownScopeResolver *ObjectResolverObject::GetTown()
  * @param spec     The specification of the object / the entry point.
  * @param o        The object to call the callback for.
  * @param tile     The tile the callback is called for.
- * @param view     The view of the object (only used when o == NULL).
+ * @param view     The view of the object (only used when o == nullptr).
  * @return The result of the callback.
  */
 uint16 GetObjectCallback(CallbackID callback, uint32 param1, uint32 param2, const ObjectSpec *spec, Object *o, TileIndex tile, uint8 view)
@@ -412,7 +419,7 @@ uint16 GetObjectCallback(CallbackID callback, uint32 param1, uint32 param2, cons
  */
 static void DrawTileLayout(const TileInfo *ti, const TileLayoutSpriteGroup *group, const ObjectSpec *spec)
 {
-	const DrawTileSprites *dts = group->ProcessRegisters(NULL);
+	const DrawTileSprites *dts = group->ProcessRegisters(nullptr);
 	PaletteID palette = ((spec->flags & OBJECT_FLAG_2CC_COLOUR) ? SPR_2CCMAP_BASE : PALETTE_RECOLOUR_START) + Object::GetByTile(ti->tile)->colour;
 
 	SpriteID image = dts->ground.sprite;
@@ -442,7 +449,7 @@ void DrawNewObjectTile(TileInfo *ti, const ObjectSpec *spec)
 	ObjectResolverObject object(spec, o, ti->tile);
 
 	const SpriteGroup *group = object.Resolve();
-	if (group == NULL || group->type != SGT_TILELAYOUT) return;
+	if (group == nullptr || group->type != SGT_TILELAYOUT) return;
 
 	DrawTileLayout(ti, (const TileLayoutSpriteGroup *)group, spec);
 }
@@ -456,11 +463,11 @@ void DrawNewObjectTile(TileInfo *ti, const ObjectSpec *spec)
  */
 void DrawNewObjectTileInGUI(int x, int y, const ObjectSpec *spec, uint8 view)
 {
-	ObjectResolverObject object(spec, NULL, INVALID_TILE, view);
+	ObjectResolverObject object(spec, nullptr, INVALID_TILE, view);
 	const SpriteGroup *group = object.Resolve();
-	if (group == NULL || group->type != SGT_TILELAYOUT) return;
+	if (group == nullptr || group->type != SGT_TILELAYOUT) return;
 
-	const DrawTileSprites *dts = ((const TileLayoutSpriteGroup *)group)->ProcessRegisters(NULL);
+	const DrawTileSprites *dts = ((const TileLayoutSpriteGroup *)group)->ProcessRegisters(nullptr);
 
 	PaletteID palette;
 	if (Company::IsValidID(_local_company)) {
@@ -518,7 +525,7 @@ struct ObjectAnimationBase : public AnimationBase<ObjectAnimationBase, ObjectSpe
 void AnimateNewObjectTile(TileIndex tile)
 {
 	const ObjectSpec *spec = ObjectSpec::GetByTile(tile);
-	if (spec == NULL || !(spec->flags & OBJECT_FLAG_ANIMATION)) return;
+	if (spec == nullptr || !(spec->flags & OBJECT_FLAG_ANIMATION)) return;
 
 	ObjectAnimationBase::AnimateTile(spec, Object::GetByTile(tile), tile, (spec->flags & OBJECT_FLAG_ANIM_RANDOM_BITS) != 0);
 }
@@ -547,7 +554,7 @@ void TriggerObjectAnimation(Object *o, ObjectAnimationTrigger trigger, const Obj
 {
 	if (!HasBit(spec->animation.triggers, trigger)) return;
 
-	TILE_AREA_LOOP(tile, o->location) {
+	for (TileIndex tile : o->location) {
 		TriggerObjectTileAnimation(o, tile, trigger, spec);
 	}
 }
