@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -33,7 +31,7 @@
 			/* Check the persistent storage for the GrfID stored in register 100h. */
 			uint32 grfid = GetRegister(0x100);
 			if (grfid == 0xFFFFFFFF) {
-				if (this->ro.grffile == NULL) return 0;
+				if (this->ro.grffile == nullptr) return 0;
 				grfid = this->ro.grffile->grfid;
 			}
 
@@ -113,7 +111,7 @@
 		case 0xD5: return this->t->fund_buildings_months;
 	}
 
-	DEBUG(grf, 1, "Unhandled town variable 0x%X", variable);
+	Debug(grf, 1, "Unhandled town variable 0x{:X}", variable);
 
 	*available = false;
 	return UINT_MAX;
@@ -123,9 +121,9 @@
 {
 	if (this->readonly) return;
 
-	assert(this->t != NULL);
+	assert(this->t != nullptr);
 	/* We can't store anything if the caller has no #GRFFile. */
-	if (this->ro.grffile == NULL) return;
+	if (this->ro.grffile == nullptr) return;
 
 	/* Check the persistent storage for the GrfID stored in register 100h. */
 	uint32 grfid = GetRegister(0x100);
