@@ -9,9 +9,15 @@
 
 #include "stdafx.h"
 #include "command_func.h"
+#include "hotkeys.h"
 #include "newgrf.h"
 #include "newgrf_object.h"
 #include "newgrf_text.h"
+#include "object.h"
+#include "querystring_gui.h"
+#include "sortlist_type.h"
+#include "stringfilter_type.h"
+#include "string_func.h"
 #include "strings_func.h"
 #include "viewport_func.h"
 #include "tilehighlight_func.h"
@@ -25,23 +31,40 @@
 
 #include "safeguards.h"
 
-static ObjectClassID _selected_object_class; ///< the currently visible object class
-static int _selected_object_index;           ///< the index of the selected object in the current class or -1
+static ObjectClassID _selected_object_class; ///< Currently selected available object class.
+static int _selected_object_index;           ///< Index of the currently selected object if existing, else \c -1.
 static uint8 _selected_object_view;          ///< the view of the selected object
 
+/** Enum referring to the Hotkeys in the build object window */
+enum BuildObjectHotkeys {
+	BOHK_FOCUS_FILTER_BOX, ///< Focus the edit box for editing the filter string
+};
+
 /** The window used for building objects. */
 class BuildObjectWindow : public Window {
-	static const int OBJECT_MARGIN = 4; ///< The margin (in pixels) around an object.
-	int line_height;                    ///< The height of a single line.
-	int info_height;                    ///< The height of the info box.
-	Scrollbar *vscroll;                 ///< The scrollbar.
+	typedef GUIList<ObjectClassID, StringFilter &> GUIObjectClassList; ///< Type definition for the list to hold available object classes.
+
+	static const uint EDITBOX_MAX_SIZE = 16; ///< The maximum number of characters for the filter edit box.
+	static const int OBJECT_MARGIN = 4;    ///< The margin (in pixels) around an object.
+
+	int line_height;                       ///< The height of a single line.
+	int info_height;                       ///< The height of the info box.
+	Scrollbar *vscroll;                    ///< The scrollbar.
+
+	static Listing   last_sorting;         ///< Default sorting of #GUIObjectClassList.
+	static Filtering last_filtering;       ///< Default filtering of #GUIObjectClassList.
+	static GUIObjectClassList::SortFunction   * const sorter_funcs[]; ///< Sort functions of the #GUIObjectClassList.
+	static GUIObjectClassList::FilterFunction * const filter_funcs[]; ///< Filter functions of the #GUIObjectClassList.
+	GUIObjectClassList object_classes;     ///< Available object classes.
+	StringFilter string_filter;            ///< Filter for available objects.
+	QueryString filter_editbox;            ///< Filter editbox.
 
 	/** Scroll #WID_BO_CLASS_LIST so that the selected object class is visible. */
 	void EnsureSelectedObjectClassIsVisible()
 	{
 		uint pos = 0;
-		for (int i = 0; i < _selected_object_class; i++) {
-			if (ObjectClass::Get((ObjectClassID) i)->GetUISpecCount() == 0) continue;
+		for (auto object_class_id : this->object_classes) {
+			if (object_class_id == _selected_object_class) break;
 			pos++;
 		}
 		this->vscroll->ScrollTowards(pos);
@@ -55,10 +78,10 @@ class BuildObjectWindow : public Window {
 	{
 		if (_selected_object_index == -1) return false;
 
-		ObjectClass *sel_objclass = ObjectClass::Get(_selected_object_class);
-		if ((int)sel_objclass->GetSpecCount() <= _selected_object_index) return false;
+		ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+		if ((int)objclass->GetSpecCount() <= _selected_object_index) return false;
 
-		return sel_objclass->GetSpec(_selected_object_index)->IsAvailable();
+		return objclass->GetSpec(_selected_object_index)->IsAvailable();
 	}
 
 	/**
@@ -72,42 +95,127 @@ class BuildObjectWindow : public Window {
 	}
 
 public:
-	BuildObjectWindow(WindowDesc *desc, WindowNumber number) : Window(desc), info_height(1)
+	BuildObjectWindow(WindowDesc *desc, WindowNumber number) : Window(desc), info_height(1), filter_editbox(EDITBOX_MAX_SIZE * MAX_CHAR_LENGTH, EDITBOX_MAX_SIZE)
 	{
 		this->CreateNestedTree();
+
 		this->vscroll = this->GetScrollbar(WID_BO_SCROLLBAR);
+
+		this->querystrings[WID_BO_FILTER] = &this->filter_editbox;
+
+		this->object_classes.SetListing(this->last_sorting);
+		this->object_classes.SetFiltering(this->last_filtering);
+		this->object_classes.SetSortFuncs(this->sorter_funcs);
+		this->object_classes.SetFilterFuncs(this->filter_funcs);
+		this->object_classes.ForceRebuild();
+
+		BuildObjectClassesAvailable();
+		SelectClassAndObject();
+
 		this->FinishInitNested(number);
 
-		ResetObjectToPlace();
-
-		this->vscroll->SetPosition(0);
-		this->vscroll->SetCount(ObjectClass::GetUIClassCount());
-
 		NWidgetMatrix *matrix = this->GetWidget<NWidgetMatrix>(WID_BO_SELECT_MATRIX);
 		matrix->SetScrollbar(this->GetScrollbar(WID_BO_SELECT_SCROLL));
 
-		this->SelectOtherClass(_selected_object_class);
+		this->GetWidget<NWidgetMatrix>(WID_BO_OBJECT_MATRIX)->SetCount(4);
+
+		ResetObjectToPlace();
+
+		this->vscroll->SetCount((int)this->object_classes.size());
+
+		EnsureSelectedObjectClassIsVisible();
+
+		this->InvalidateData();
+	}
+
+	/** Sort object classes by ObjectClassID. */
+	static bool ObjectClassIDSorter(ObjectClassID const &a, ObjectClassID const &b)
+	{
+		return a < b;
+	}
+
+	/** Filter object classes by class name. */
+	static bool CDECL TagNameFilter(ObjectClassID const *oc, StringFilter &filter)
+	{
+		ObjectClass *objclass = ObjectClass::Get(*oc);
+		char buffer[DRAW_STRING_BUFFER];
+		GetString(buffer, objclass->name, lastof(buffer));
+
+		filter.ResetState();
+		filter.AddLine(buffer);
+		return filter.GetState();
+	}
+
+	/** Builds the filter list of available object classes. */
+	void BuildObjectClassesAvailable()
+	{
+		if (!this->object_classes.NeedRebuild()) return;
+
+		this->object_classes.clear();
+
+		for (uint i = 0; i < ObjectClass::GetClassCount(); i++) {
+			ObjectClass *objclass = ObjectClass::Get((ObjectClassID)i);
+			if (objclass->GetUISpecCount() == 0) continue; // Is this needed here?
+			object_classes.push_back((ObjectClassID)i);
+		}
+
+		this->object_classes.Filter(this->string_filter);
+		this->object_classes.shrink_to_fit();
+		this->object_classes.RebuildDone();
+		this->object_classes.Sort();
+
+		this->vscroll->SetCount((uint)this->object_classes.size());
+	}
+
+	/**
+	 * Checks if the previously selected current object class and object
+	 * can be shown as selected to the user when the dialog is opened.
+	 */
+	void SelectClassAndObject()
+	{
+		assert(!this->object_classes.empty()); // object GUI should be disabled elsewise
+		if (_selected_object_class == ObjectClassID::OBJECT_CLASS_BEGIN) {
+			/* This happens during the first time the window is open during the game life cycle. */
+			this->SelectOtherClass(this->object_classes[0]);
+		} else {
+			/* Check if the previously selected object class is not available anymore as a
+			 * result of starting a new game without the corresponding NewGRF. */
+			bool available = false;
+			for (uint i = 0; ObjectClass::GetClassCount(); ++i) {
+				if ((ObjectClassID)i == _selected_object_class) {
+					available = true;
+					break;
+				}
+			}
+
+			if (available) {
+				this->SelectOtherClass(_selected_object_class);
+			} else {
+				this->SelectOtherClass(this->object_classes[0]);
+			}
+		}
+
 		if (this->CanRestoreSelectedObject()) {
 			this->SelectOtherObject(_selected_object_index);
 		} else {
 			this->SelectFirstAvailableObject(true);
 		}
-		assert(ObjectClass::Get(_selected_object_class)->GetUISpecCount() > 0); // object GUI should be disables elsewise
-		this->EnsureSelectedObjectClassIsVisible();
-		this->GetWidget<NWidgetMatrix>(WID_BO_OBJECT_MATRIX)->SetCount(4);
+		assert(ObjectClass::Get(_selected_object_class)->GetUISpecCount() > 0); // object GUI should be disabled elsewise
 	}
 
 	void SetStringParameters(int widget) const override
 	{
 		switch (widget) {
 			case WID_BO_OBJECT_NAME: {
-				const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
+				ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+				const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
 				SetDParam(0, spec != nullptr ? spec->name : STR_EMPTY);
 				break;
 			}
 
 			case WID_BO_OBJECT_SIZE: {
-				const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
+				ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+				const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
 				int size = spec == nullptr ? 0 : spec->size;
 				SetDParam(0, GB(size, HasBit(_selected_object_view, 0) ? 4 : 0, 4));
 				SetDParam(1, GB(size, HasBit(_selected_object_view, 0) ? 0 : 4, 4));
@@ -122,10 +230,10 @@ public:
 	{
 		switch (widget) {
 			case WID_BO_CLASS_LIST: {
-				for (uint i = 0; i < ObjectClass::GetClassCount(); i++) {
-					ObjectClass *objclass = ObjectClass::Get((ObjectClassID)i);
+				for (auto object_class_id : this->object_classes) {
+					ObjectClass *objclass = ObjectClass::Get(object_class_id);
 					if (objclass->GetUISpecCount() == 0) continue;
-					size->width = max(size->width, GetStringBoundingBox(objclass->name).width);
+					size->width = std::max(size->width, GetStringBoundingBox(objclass->name).width);
 				}
 				size->width += padding.width;
 				this->line_height = FONT_HEIGHT_NORMAL + WD_MATRIX_TOP + WD_MATRIX_BOTTOM;
@@ -142,7 +250,8 @@ public:
 
 			case WID_BO_OBJECT_MATRIX: {
 				/* Get the right amount of buttons based on the current spec. */
-				const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
+				ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+				const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
 				if (spec != nullptr) {
 					if (spec->views >= 2) size->width  += resize->width;
 					if (spec->views >= 4) size->height += resize->height;
@@ -161,7 +270,7 @@ public:
 					const ObjectSpec *spec = ObjectSpec::Get(i);
 					if (!spec->IsEverAvailable()) continue;
 					two_wide |= spec->views >= 2;
-					height[spec->views / 4] = max<int>(ObjectSpec::Get(i)->height, height[spec->views / 4]);
+					height[spec->views / 4] = std::max<int>(ObjectSpec::Get(i)->height, height[spec->views / 4]);
 				}
 
 				/* Determine the pixel heights. */
@@ -174,7 +283,7 @@ public:
 				 * we want these columns to be slightly less wide. When there are two rows, then
 				 * determine the size of the widgets based on the maximum size for a single row
 				 * of widgets, or just the twice the widget height of the two row ones. */
-				size->height = max(height[0], height[1] * 2 + 2);
+				size->height = std::max(height[0], height[1] * 2 + 2);
 				if (two_wide) {
 					size->width  = (3 * ScaleGUITrad(TILE_PIXELS) + 2 * OBJECT_MARGIN) * 2 + 2;
 				} else {
@@ -182,7 +291,8 @@ public:
 				}
 
 				/* Get the right size for the single widget based on the current spec. */
-				const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
+				ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+				const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
 				if (spec != nullptr) {
 					if (spec->views >= 2) size->width  = size->width  / 2 - 1;
 					if (spec->views >= 4) size->height = size->height / 2 - 1;
@@ -214,19 +324,22 @@ public:
 			case WID_BO_CLASS_LIST: {
 				int y = r.top;
 				uint pos = 0;
-				for (uint i = 0; i < ObjectClass::GetClassCount(); i++) {
-					ObjectClass *objclass = ObjectClass::Get((ObjectClassID)i);
+				for (auto object_class_id : this->object_classes) {
+					ObjectClass *objclass = ObjectClass::Get(object_class_id);
 					if (objclass->GetUISpecCount() == 0) continue;
 					if (!this->vscroll->IsVisible(pos++)) continue;
 					DrawString(r.left + WD_MATRIX_LEFT, r.right - WD_MATRIX_RIGHT, y + WD_MATRIX_TOP, objclass->name,
-							((int)i == _selected_object_class) ? TC_WHITE : TC_BLACK);
+							(object_class_id == _selected_object_class) ? TC_WHITE : TC_BLACK);
 					y += this->line_height;
 				}
 				break;
 			}
 
 			case WID_BO_OBJECT_SPRITE: {
-				const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
+				if (_selected_object_index == -1) break;
+
+				ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+				const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
 				if (spec == nullptr) break;
 
 				/* Height of the selection matrix.
@@ -274,7 +387,7 @@ public:
 						DrawOrigTileSeqInGUI((r.right - r.left) / 2 - 1, r.bottom - r.top - OBJECT_MARGIN - ScaleGUITrad(TILE_PIXELS), dts, PAL_NONE);
 					} else {
 						DrawNewObjectTileInGUI((r.right - r.left) / 2 - 1, r.bottom - r.top - OBJECT_MARGIN - ScaleGUITrad(TILE_PIXELS), spec,
-								min(_selected_object_view, spec->views - 1));
+								std::min<int>(_selected_object_view, spec->views - 1));
 					}
 					_cur_dpi = old_dpi;
 				}
@@ -282,7 +395,8 @@ public:
 			}
 
 			case WID_BO_INFO: {
-				const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
+				ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+				const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
 				if (spec == nullptr) break;
 
 				/* Get the extra message for the GUI */
@@ -315,12 +429,13 @@ public:
 
 	/**
 	 * Select the specified object class.
-	 * @param object_class_index Object class index to select.
+	 * @param object_class Object class select.
 	 */
-	void SelectOtherClass(ObjectClassID object_class_index)
+	void SelectOtherClass(ObjectClassID object_class)
 	{
-		_selected_object_class = object_class_index;
-		this->GetWidget<NWidgetMatrix>(WID_BO_SELECT_MATRIX)->SetCount(ObjectClass::Get(_selected_object_class)->GetUISpecCount());
+		_selected_object_class = object_class;
+		ObjectClass *objclass = ObjectClass::Get(object_class);
+		this->GetWidget<NWidgetMatrix>(WID_BO_SELECT_MATRIX)->SetCount(objclass->GetUISpecCount());
 	}
 
 	/**
@@ -331,8 +446,9 @@ public:
 	{
 		_selected_object_index = object_index;
 		if (_selected_object_index != -1) {
-			const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
-			_selected_object_view = min(_selected_object_view, spec->views - 1);
+			ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+			const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
+			_selected_object_view = std::min<int>(_selected_object_view, spec->views - 1);
 			this->ReInit();
 		} else {
 			_selected_object_view = 0;
@@ -350,7 +466,8 @@ public:
 		if (_selected_object_index == -1) {
 			SetTileSelectSize(1, 1);
 		} else {
-			const ObjectSpec *spec = ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index);
+			ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+			const ObjectSpec *spec = objclass->GetSpec(_selected_object_index);
 			int w = GB(spec->size, HasBit(_selected_object_view, 0) ? 4 : 0, 4);
 			int h = GB(spec->size, HasBit(_selected_object_view, 0) ? 0 : 4, 4);
 			SetTileSelectSize(w, h);
@@ -359,11 +476,11 @@ public:
 
 	/**
 	 * Update buttons to show the selection to the user.
-	 * @param sel_class The class of the selected object.
+	 * @param object_class The class of the selected object.
 	 * @param sel_index Index of the object to select, or \c -1 .
 	 * @param sel_view View of the object to select.
 	 */
-	void UpdateButtons(ObjectClassID sel_class, int sel_index, uint sel_view)
+	void UpdateButtons(ObjectClassID object_class, int sel_index, uint sel_view)
 	{
 		int view_number, object_number;
 		if (sel_index == -1) {
@@ -371,7 +488,8 @@ public:
 			object_number = -1;
 		} else {
 			view_number = sel_view;
-			object_number = ObjectClass::Get(sel_class)->GetUIFromIndex(sel_index);
+			ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+			object_number = objclass->GetUIFromIndex(sel_index);
 		}
 
 		this->GetWidget<NWidgetMatrix>(WID_BO_OBJECT_MATRIX)->SetClicked(view_number);
@@ -380,6 +498,13 @@ public:
 		this->SetDirty();
 	}
 
+	void OnInvalidateData(int data = 0, bool gui_scope = true) override
+	{
+		if (!gui_scope) return;
+
+		this->BuildObjectClassesAvailable();
+	}
+
 	void OnResize() override
 	{
 		this->vscroll->SetCapacityFromWidget(this, WID_BO_CLASS_LIST);
@@ -389,10 +514,10 @@ public:
 	{
 		switch (GB(widget, 0, 16)) {
 			case WID_BO_CLASS_LIST: {
-				int num_clicked = this->vscroll->GetPosition() + (pt.y - this->nested_array[widget]->pos_y) / this->line_height;
-				if (num_clicked >= (int)ObjectClass::GetUIClassCount()) break;
+				int num_clicked = this->vscroll->GetPosition() + (pt.y - this->GetWidget<NWidgetBase>(widget)->pos_y) / this->line_height;
+				if (num_clicked >= (int)this->object_classes.size()) break;
 
-				this->SelectOtherClass(ObjectClass::GetUIClass(num_clicked));
+				this->SelectOtherClass(this->object_classes[num_clicked]);
 				this->SelectFirstAvailableObject(false);
 				break;
 			}
@@ -415,7 +540,8 @@ public:
 
 	void OnPlaceObject(Point pt, TileIndex tile) override
 	{
-		DoCommandP(tile, ObjectClass::Get(_selected_object_class)->GetSpec(_selected_object_index)->Index(),
+		ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+		DoCommandP(tile, objclass->GetSpec(_selected_object_index)->Index(),
 				_selected_object_view, CMD_BUILD_OBJECT | CMD_MSG(STR_ERROR_CAN_T_BUILD_OBJECT), CcTerraform);
 	}
 
@@ -424,6 +550,29 @@ public:
 		this->UpdateButtons(_selected_object_class, -1, _selected_object_view);
 	}
 
+	EventState OnHotkey(int hotkey) override
+	{
+		switch (hotkey) {
+			case BOHK_FOCUS_FILTER_BOX:
+				this->SetFocusedWidget(WID_BO_FILTER);
+				SetFocusedWindow(this); // The user has asked to give focus to the text box, so make sure this window is focused.
+				break;
+
+			default:
+				return ES_NOT_HANDLED;
+		}
+
+		return ES_HANDLED;
+	}
+
+	void OnEditboxChanged(int wid) override
+	{
+		string_filter.SetFilterTerm(this->filter_editbox.text.buf);
+		this->object_classes.SetFilterState(!string_filter.IsEmpty());
+		this->object_classes.ForceRebuild();
+		this->InvalidateData();
+	}
+
 	/**
 	 * Select the first available object.
 	 * @param change_class If true, change the class if no object in the current
@@ -431,10 +580,11 @@ public:
 	 */
 	void SelectFirstAvailableObject(bool change_class)
 	{
+		ObjectClass *objclass = ObjectClass::Get(_selected_object_class);
+
 		/* First try to select an object in the selected class. */
-		ObjectClass *sel_objclass = ObjectClass::Get(_selected_object_class);
-		for (uint i = 0; i < sel_objclass->GetSpecCount(); i++) {
-			const ObjectSpec *spec = sel_objclass->GetSpec(i);
+		for (uint i = 0; i < objclass->GetSpecCount(); i++) {
+			const ObjectSpec *spec = objclass->GetSpec(i);
 			if (spec->IsAvailable()) {
 				this->SelectOtherObject(i);
 				return;
@@ -443,12 +593,12 @@ public:
 		if (change_class) {
 			/* If that fails, select the first available object
 			 * from a random class. */
-			for (ObjectClassID j = OBJECT_CLASS_BEGIN; j < OBJECT_CLASS_MAX; j++) {
-				ObjectClass *objclass = ObjectClass::Get(j);
+			for (auto object_class_id : this->object_classes) {
+				ObjectClass *objclass = ObjectClass::Get(object_class_id);
 				for (uint i = 0; i < objclass->GetSpecCount(); i++) {
 					const ObjectSpec *spec = objclass->GetSpec(i);
 					if (spec->IsAvailable()) {
-						this->SelectOtherClass(j);
+						this->SelectOtherClass(object_class_id);
 						this->SelectOtherObject(i);
 						return;
 					}
@@ -456,17 +606,50 @@ public:
 			}
 		}
 		/* If all objects are unavailable, select nothing... */
-		if (ObjectClass::Get(_selected_object_class)->GetUISpecCount() == 0) {
+		if (objclass->GetUISpecCount() == 0) {
 			/* ... but make sure that the class is not empty. */
-			for (ObjectClassID j = OBJECT_CLASS_BEGIN; j < OBJECT_CLASS_MAX; j++) {
-				if (ObjectClass::Get(j)->GetUISpecCount() > 0) {
-					this->SelectOtherClass(j);
+			for (auto object_class_id : this->object_classes) {
+				ObjectClass *objclass = ObjectClass::Get(object_class_id);
+				if (objclass->GetUISpecCount() > 0) {
+					this->SelectOtherClass(object_class_id);
 					break;
 				}
 			}
 		}
 		this->SelectOtherObject(-1);
 	}
+
+	static HotkeyList hotkeys;
+};
+
+/**
+ * Handler for global hotkeys of the BuildObjectWindow.
+ * @param hotkey Hotkey
+ * @return ES_HANDLED if hotkey was accepted.
+ */
+static EventState BuildObjectGlobalHotkeys(int hotkey)
+{
+	if (_game_mode == GM_MENU) return ES_NOT_HANDLED;
+	Window *w = ShowBuildObjectPicker();
+	if (w == nullptr) return ES_NOT_HANDLED;
+	return w->OnHotkey(hotkey);
+}
+
+static Hotkey buildobject_hotkeys[] = {
+	Hotkey('F', "focus_filter_box", BOHK_FOCUS_FILTER_BOX),
+	HOTKEY_LIST_END
+};
+HotkeyList BuildObjectWindow::hotkeys("buildobject", buildobject_hotkeys, BuildObjectGlobalHotkeys);
+
+Listing BuildObjectWindow::last_sorting = { false, 0 };
+Filtering BuildObjectWindow::last_filtering = { false, 0 };
+
+BuildObjectWindow::GUIObjectClassList::SortFunction * const BuildObjectWindow::sorter_funcs[] = {
+	&ObjectClassIDSorter,
+};
+
+BuildObjectWindow::GUIObjectClassList::FilterFunction * const BuildObjectWindow::filter_funcs[] = {
+	&TagNameFilter,
 };
 
 static const NWidgetPart _nested_build_object_widgets[] = {
@@ -476,19 +659,24 @@ static const NWidgetPart _nested_build_object_widgets[] = {
 		NWidget(WWT_DEFSIZEBOX, COLOUR_DARK_GREEN),
 	EndContainer(),
 	NWidget(WWT_PANEL, COLOUR_DARK_GREEN),
-		NWidget(NWID_HORIZONTAL), SetPadding(2, 0, 0, 0),
-			NWidget(NWID_VERTICAL),
-				NWidget(NWID_HORIZONTAL), SetPadding(0, 5, 2, 5),
+		NWidget(NWID_HORIZONTAL), SetPadding(2, 0, 0, 2),
+			NWidget(NWID_VERTICAL), SetPadding(0, 5, 2, 0), SetPIP(0, 2, 0),
+				NWidget(NWID_HORIZONTAL),
+					NWidget(WWT_TEXT, COLOUR_DARK_GREEN), SetFill(0, 1), SetDataTip(STR_LIST_FILTER_TITLE, STR_NULL),
+					NWidget(WWT_EDITBOX, COLOUR_GREY, WID_BO_FILTER), SetFill(1, 0), SetResize(1, 0),
+							SetDataTip(STR_LIST_FILTER_OSKTITLE, STR_LIST_FILTER_TOOLTIP),
+				EndContainer(),
+				NWidget(NWID_HORIZONTAL),
 					NWidget(WWT_MATRIX, COLOUR_GREY, WID_BO_CLASS_LIST), SetFill(1, 0), SetMatrixDataTip(1, 0, STR_OBJECT_BUILD_CLASS_TOOLTIP), SetScrollbar(WID_BO_SCROLLBAR),
 					NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_BO_SCROLLBAR),
 				EndContainer(),
-				NWidget(NWID_HORIZONTAL), SetPadding(0, 5, 0, 5),
+				NWidget(NWID_HORIZONTAL),
 					NWidget(NWID_MATRIX, COLOUR_DARK_GREEN, WID_BO_OBJECT_MATRIX), SetPIP(0, 2, 0),
 						NWidget(WWT_PANEL, COLOUR_GREY, WID_BO_OBJECT_SPRITE), SetDataTip(0x0, STR_OBJECT_BUILD_PREVIEW_TOOLTIP), EndContainer(),
 					EndContainer(),
 				EndContainer(),
-				NWidget(WWT_TEXT, COLOUR_DARK_GREEN, WID_BO_OBJECT_NAME), SetDataTip(STR_ORANGE_STRING, STR_NULL), SetPadding(2, 5, 2, 5),
-				NWidget(WWT_TEXT, COLOUR_DARK_GREEN, WID_BO_OBJECT_SIZE), SetDataTip(STR_OBJECT_BUILD_SIZE, STR_NULL), SetPadding(2, 5, 2, 5),
+				NWidget(WWT_TEXT, COLOUR_DARK_GREEN, WID_BO_OBJECT_NAME), SetDataTip(STR_ORANGE_STRING, STR_NULL),
+				NWidget(WWT_TEXT, COLOUR_DARK_GREEN, WID_BO_OBJECT_SIZE), SetDataTip(STR_OBJECT_BUILD_SIZE, STR_NULL),
 			EndContainer(),
 			NWidget(WWT_PANEL, COLOUR_DARK_GREEN), SetScrollbar(WID_BO_SELECT_SCROLL),
 				NWidget(NWID_HORIZONTAL),
@@ -502,7 +690,7 @@ static const NWidgetPart _nested_build_object_widgets[] = {
 			EndContainer(),
 		EndContainer(),
 		NWidget(NWID_HORIZONTAL),
-			NWidget(WWT_EMPTY, INVALID_COLOUR, WID_BO_INFO), SetPadding(2, 5, 0, 5), SetFill(1, 0), SetResize(1, 0),
+			NWidget(WWT_EMPTY, INVALID_COLOUR, WID_BO_INFO), SetPadding(0, 5, 0, 1), SetFill(1, 0), SetResize(1, 0),
 			NWidget(NWID_VERTICAL),
 				NWidget(WWT_PANEL, COLOUR_DARK_GREEN), SetFill(0, 1), EndContainer(),
 				NWidget(WWT_RESIZEBOX, COLOUR_DARK_GREEN),
@@ -515,20 +703,22 @@ static WindowDesc _build_object_desc(
 	WDP_AUTO, "build_object", 0, 0,
 	WC_BUILD_OBJECT, WC_BUILD_TOOLBAR,
 	WDF_CONSTRUCTION,
-	_nested_build_object_widgets, lengthof(_nested_build_object_widgets)
+	_nested_build_object_widgets, lengthof(_nested_build_object_widgets),
+	&BuildObjectWindow::hotkeys
 );
 
 /** Show our object picker.  */
-void ShowBuildObjectPicker()
+Window *ShowBuildObjectPicker()
 {
 	/* Don't show the place object button when there are no objects to place. */
 	if (ObjectClass::GetUIClassCount() > 0) {
-		AllocateWindowDescFront<BuildObjectWindow>(&_build_object_desc, 0);
+		return AllocateWindowDescFront<BuildObjectWindow>(&_build_object_desc, 0);
 	}
+	return nullptr;
 }
 
 /** Reset all data of the object GUI. */
 void InitializeObjectGui()
 {
-	_selected_object_class = (ObjectClassID)0;
+	_selected_object_class = ObjectClassID::OBJECT_CLASS_BEGIN;
 }
