@@ -18,6 +18,7 @@
 #include "station_type.h"
 #include "vehicle_type.h"
 #include "date_type.h"
+#include "saveload/saveload.h"
 
 typedef Pool<Order, OrderID, 256, 0xFF0000> OrderPool;
 typedef Pool<OrderList, OrderListID, 128, 64000> OrderListPool;
@@ -31,9 +32,11 @@ extern OrderListPool _orderlist_pool;
  */
 struct Order : OrderPool::PoolItem<&_order_pool> {
 private:
-	friend const struct SaveLoad *GetVehicleDescription(VehicleType vt); ///< Saving and loading the current order of vehicles.
-	friend void Load_VEHS();                                             ///< Loading of ancient vehicles.
-	friend const struct SaveLoad *GetOrderDescription();                 ///< Saving and loading of orders.
+	friend struct VEHSChunkHandler;                             ///< Loading of ancient vehicles.
+	friend SaveLoadTable GetOrderDescription();                 ///< Saving and loading of orders.
+	/* So we can use private/protected variables in the saveload code */
+	friend class SlVehicleCommon;
+	friend class SlVehicleDisaster;
 
 	uint8 type;           ///< The type of order + non-stop flags
 	uint8 flags;          ///< Load/unload types, depot order/action types.
@@ -250,7 +253,7 @@ void DeleteOrder(Vehicle *v, VehicleOrderID sel_ord);
 struct OrderList : OrderListPool::PoolItem<&_orderlist_pool> {
 private:
 	friend void AfterLoadVehicles(bool part_of_load); ///< For instantiating the shared vehicle chain
-	friend const struct SaveLoad *GetOrderListDescription(); ///< Saving and loading of order lists.
+	friend SaveLoadTable GetOrderListDescription(); ///< Saving and loading of order lists.
 
 	StationID GetBestLoadableNext(const Vehicle *v, const Order *o1, const Order *o2) const;
 
@@ -392,6 +395,4 @@ public:
 	void DebugCheckSanity() const;
 };
 
-#define FOR_VEHICLE_ORDERS(v, order) for (order = (v->orders.list == nullptr) ? nullptr : v->orders.list->GetFirstOrder(); order != nullptr; order = order->next)
-
 #endif /* ORDER_BASE_H */
