@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -33,8 +31,8 @@ static const OrderID INVALID_ORDER = 0xFFFFFF;
  */
 static const uint IMPLICIT_ORDER_ONLY_CAP = 32;
 
-/** Order types */
-enum OrderType {
+/** Order types. It needs to be 8bits, because we save and load it as such */
+enum OrderType : byte {
 	OT_BEGIN         = 0,
 	OT_NOTHING       = 0,
 	OT_GOTO_STATION  = 1,
@@ -44,14 +42,10 @@ enum OrderType {
 	OT_DUMMY         = 5,
 	OT_GOTO_WAYPOINT = 6,
 	OT_CONDITIONAL   = 7,
-	OT_IMPLICIT     = 8,
+	OT_IMPLICIT      = 8,
 	OT_END
 };
 
-/** It needs to be 8bits, because we save and load it as such */
-typedef SimpleTinyEnumT<OrderType, byte> OrderTypeByte;
-
-
 /**
  * Flags related to the unloading order.
  */
