@@ -63,10 +63,12 @@ class CrashLogOSX : public CrashLog {
 				" Name:     Mac OS X\n"
 				" Release:  %d.%d.%d\n"
 				" Machine:  %s\n"
-				" Min Ver:  %d\n",
+				" Min Ver:  %d\n"
+				" Max Ver:  %d\n",
 				ver_maj, ver_min, ver_bug,
 				arch != nullptr ? arch->description : "unknown",
-				MAC_OS_X_VERSION_MIN_REQUIRED
+				MAC_OS_X_VERSION_MIN_REQUIRED,
+				MAC_OS_X_VERSION_MAX_ALLOWED
 		);
 	}
 
@@ -255,3 +257,7 @@ void CDECL HandleCrash(int signum)
 		signal(*i, HandleCrash);
 	}
 }
+
+/* static */ void CrashLog::InitThread()
+{
+}
