@@ -8,9 +8,11 @@
 /** @file story_sl.cpp Code handling saving and loading of story pages */
 
 #include "../stdafx.h"
-#include "../story_base.h"
 
 #include "saveload.h"
+#include "compat/story_sl_compat.h"
+
+#include "../story_base.h"
 
 #include "../safeguards.h"
 
@@ -34,33 +36,40 @@ static const SaveLoad _story_page_elements_desc[] = {
 	SLE_CONDVAR(StoryPageElement, type,          SLE_UINT8,                  SLV_185, SL_MAX_VERSION),
 	    SLE_VAR(StoryPageElement, referenced_id, SLE_UINT32),
 	    SLE_STR(StoryPageElement, text,          SLE_STR | SLF_ALLOW_CONTROL, 0),
-	    SLE_END()
 };
 
-static void Save_STORY_PAGE_ELEMENT()
-{
-	for (StoryPageElement *s : StoryPageElement::Iterate()) {
-		SlSetArrayIndex(s->index);
-		SlObject(s, _story_page_elements_desc);
-	}
-}
+struct STPEChunkHandler : ChunkHandler {
+	STPEChunkHandler() : ChunkHandler('STPE', CH_TABLE) {}
 
-static void Load_STORY_PAGE_ELEMENT()
-{
-	int index;
-	uint32 max_sort_value = 0;
-	while ((index = SlIterateArray()) != -1) {
-		StoryPageElement *s = new (index) StoryPageElement();
-		SlObject(s, _story_page_elements_desc);
-		if (s->sort_value > max_sort_value) {
-			max_sort_value = s->sort_value;
+	void Save() const override
+	{
+		SlTableHeader(_story_page_elements_desc);
+
+		for (StoryPageElement *s : StoryPageElement::Iterate()) {
+			SlSetArrayIndex(s->index);
+			SlObject(s, _story_page_elements_desc);
 		}
 	}
-	/* Update the next sort value, so that the next
-	 * created page is shown after all existing pages.
-	 */
-	_story_page_element_next_sort_value = max_sort_value + 1;
-}
+
+	void Load() const override
+	{
+		const std::vector<SaveLoad> slt = SlCompatTableHeader(_story_page_elements_desc, _story_page_elements_sl_compat);
+
+		int index;
+		uint32 max_sort_value = 0;
+		while ((index = SlIterateArray()) != -1) {
+			StoryPageElement *s = new (index) StoryPageElement();
+			SlObject(s, slt);
+			if (s->sort_value > max_sort_value) {
+				max_sort_value = s->sort_value;
+			}
+		}
+		/* Update the next sort value, so that the next
+		 * created page is shown after all existing pages.
+		 */
+		_story_page_element_next_sort_value = max_sort_value + 1;
+	}
+};
 
 static const SaveLoad _story_pages_desc[] = {
 	SLE_CONDVAR(StoryPage, sort_value, SLE_FILE_U16 | SLE_VAR_U32, SL_MIN_VERSION,   SLV_185),
@@ -69,35 +78,46 @@ static const SaveLoad _story_pages_desc[] = {
 	SLE_CONDVAR(StoryPage, company,    SLE_FILE_U16 | SLE_VAR_U8,  SL_MIN_VERSION,   SLV_185),
 	SLE_CONDVAR(StoryPage, company,    SLE_UINT8,                  SLV_185, SL_MAX_VERSION),
 	    SLE_STR(StoryPage, title,      SLE_STR | SLF_ALLOW_CONTROL, 0),
-	    SLE_END()
 };
 
-static void Save_STORY_PAGE()
-{
-	for (StoryPage *s : StoryPage::Iterate()) {
-		SlSetArrayIndex(s->index);
-		SlObject(s, _story_pages_desc);
-	}
-}
+struct STPAChunkHandler : ChunkHandler {
+	STPAChunkHandler() : ChunkHandler('STPA', CH_TABLE) {}
 
-static void Load_STORY_PAGE()
-{
-	int index;
-	uint32 max_sort_value = 0;
-	while ((index = SlIterateArray()) != -1) {
-		StoryPage *s = new (index) StoryPage();
-		SlObject(s, _story_pages_desc);
-		if (s->sort_value > max_sort_value) {
-			max_sort_value = s->sort_value;
+	void Save() const override
+	{
+		SlTableHeader(_story_pages_desc);
+
+		for (StoryPage *s : StoryPage::Iterate()) {
+			SlSetArrayIndex(s->index);
+			SlObject(s, _story_pages_desc);
 		}
 	}
-	/* Update the next sort value, so that the next
-	 * created page is shown after all existing pages.
-	 */
-	_story_page_next_sort_value = max_sort_value + 1;
-}
 
-extern const ChunkHandler _story_page_chunk_handlers[] = {
-	{ 'STPE', Save_STORY_PAGE_ELEMENT, Load_STORY_PAGE_ELEMENT, nullptr, nullptr, CH_ARRAY},
-	{ 'STPA', Save_STORY_PAGE,         Load_STORY_PAGE,         nullptr, nullptr, CH_ARRAY | CH_LAST},
+	void Load() const override
+	{
+		const std::vector<SaveLoad> slt = SlCompatTableHeader(_story_pages_desc, _story_pages_sl_compat);
+
+		int index;
+		uint32 max_sort_value = 0;
+		while ((index = SlIterateArray()) != -1) {
+			StoryPage *s = new (index) StoryPage();
+			SlObject(s, slt);
+			if (s->sort_value > max_sort_value) {
+				max_sort_value = s->sort_value;
+			}
+		}
+		/* Update the next sort value, so that the next
+		 * created page is shown after all existing pages.
+		 */
+		_story_page_next_sort_value = max_sort_value + 1;
+	}
 };
+
+static const STPEChunkHandler STPE;
+static const STPAChunkHandler STPA;
+static const ChunkHandlerRef story_page_chunk_handlers[] = {
+	STPE,
+	STPA,
+};
+
+extern const ChunkHandlerTable _story_page_chunk_handlers(story_page_chunk_handlers);
