@@ -11,7 +11,9 @@
 #include "script_cargo.hpp"
 #include "../../economy_func.h"
 #include "../../core/bitmath_func.hpp"
+#include "../../strings_func.h"
 #include "../../settings_type.h"
+#include "table/strings.h"
 
 #include "../../safeguards.h"
 
@@ -25,6 +27,14 @@
 	return (towneffect_type >= (TownEffect)TE_BEGIN && towneffect_type < (TownEffect)TE_END);
 }
 
+/* static */ char *ScriptCargo::GetName(CargoID cargo_type)
+{
+	if (!IsValidCargo(cargo_type)) return nullptr;
+
+	::SetDParam(0, 1ULL << cargo_type);
+	return GetString(STR_JUST_CARGO_LIST);
+}
+
 /* static */ char *ScriptCargo::GetCargoLabel(CargoID cargo_type)
 {
 	if (!IsValidCargo(cargo_type)) return nullptr;
@@ -70,4 +80,4 @@
 {
 	if (!ScriptCargo::IsValidCargo(cargo_type)) return INVALID_DISTRIBUTION_TYPE;
 	return (ScriptCargo::DistributionType)_settings_game.linkgraph.GetDistributionType(cargo_type);
-}
\ No newline at end of file
+}
