@@ -54,6 +54,9 @@ public:
 		ET_EXCLUSIVE_TRANSPORT_RIGHTS,
 		ET_ROAD_RECONSTRUCTION,
 		ET_VEHICLE_AUTOREPLACED,
+		ET_STORYPAGE_BUTTON_CLICK,
+		ET_STORYPAGE_TILE_SELECT,
+		ET_STORYPAGE_VEHICLE_SELECT,
 	};
 
 	/**
