@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -799,8 +797,8 @@ private:
 
 /**
  * Event AircraftDestTooFar, indicating the next destination of an aircraft is too far away.
- * This event can be trigger when the current oder of an aircraft changes, usually either when
- * loading is done or when switch manually.
+ * This event can be triggered when the current order of an aircraft changes, usually either when
+ * loading is done or when switched manually.
  * @api ai
  */
 class ScriptEventAircraftDestTooFar : public ScriptEvent {
@@ -839,8 +837,7 @@ public:
 	/**
 	 * @param json The JSON string which got sent.
 	 */
-	ScriptEventAdminPort(const char *json);
-	~ScriptEventAdminPort();
+	ScriptEventAdminPort(const std::string &json);
 
 	/**
 	 * Convert an ScriptEvent to the real instance.
@@ -855,28 +852,28 @@ public:
 	SQInteger GetObject(HSQUIRRELVM vm);
 
 private:
-	char *json; ///< The JSON string.
+	std::string json; ///< The JSON string.
 
 	/**
 	 * Read a table from a JSON string.
 	 * @param vm The VM used.
 	 * @param p The (part of the) JSON string reading.
 	 */
-	char *ReadTable(HSQUIRRELVM vm, char *p);
+	const char *ReadTable(HSQUIRRELVM vm, const char *p);
 
 	/**
 	 * Read a value from a JSON string.
 	 * @param vm The VM used.
 	 * @param p The (part of the) JSON string reading.
 	 */
-	char *ReadValue(HSQUIRRELVM vm, char *p);
+	const char *ReadValue(HSQUIRRELVM vm, const char *p);
 
 	/**
 	 * Read a string from a JSON string.
 	 * @param vm The VM used.
 	 * @param p The (part of the) JSON string reading.
 	 */
-	char *ReadString(HSQUIRRELVM vm, char *p);
+	const char *ReadString(HSQUIRRELVM vm, const char *p);
 };
 
 /**
@@ -970,7 +967,7 @@ public:
 private:
 	uint16 uniqueid;                   ///< The uniqueid of the question.
 	ScriptCompany::CompanyID company;  ///< The company given the answer.
-	ScriptGoal::QuestionButton button; ///< The button he pressed.
+	ScriptGoal::QuestionButton button; ///< The button that was pressed.
 };
 
 /**
@@ -1060,4 +1057,176 @@ public:
 	static ScriptEventRoadReconstruction *Convert(ScriptEventCompanyTown *instance) { return (ScriptEventRoadReconstruction *)instance; }
 };
 
+/**
+ * Event VehicleAutoReplaced, indicating a vehicle has been auto replaced.
+ * @api ai
+ */
+class ScriptEventVehicleAutoReplaced : public ScriptEvent {
+public:
+	/**
+	 * @param old_id The vehicle that has been replaced.
+	 * @param new_id The vehicle that has been created in replacement.
+	 */
+	ScriptEventVehicleAutoReplaced(VehicleID old_id, VehicleID new_id) :
+		ScriptEvent(ET_VEHICLE_AUTOREPLACED),
+		old_id(old_id),
+		new_id(new_id)
+	{}
+
+	/**
+	 * Convert an ScriptEvent to the real instance.
+	 * @param instance The instance to convert.
+	 * @return The converted instance.
+	 */
+	static ScriptEventVehicleAutoReplaced *Convert(ScriptEvent *instance) { return (ScriptEventVehicleAutoReplaced *)instance; }
+
+	/**
+	 * Get the VehicleID of the vehicle that has been replaced.
+	 * @return The VehicleID of the vehicle that has been replaced. This ID is no longer valid for referencing the vehicle.
+	 */
+	VehicleID GetOldVehicleID() { return this->old_id; }
+
+	/**
+	 * Get the VehicleID of the vehicle that has been created in replacement.
+	 * @return The VehicleID of the vehicle that has been created in replacement.
+	 */
+	VehicleID GetNewVehicleID() { return this->new_id; }
+
+private:
+	VehicleID old_id; ///< The vehicle that has been replaced.
+	VehicleID new_id; ///< The vehicle that has been created in replacement.
+};
+
+/**
+ * Event StoryPageButtonClick, indicating a player clicked a push button on a storybook page.
+ * @api game
+ */
+class ScriptEventStoryPageButtonClick : public ScriptEvent {
+public:
+	/**
+	 * @param company_id  Which company triggered the event.
+	 * @param page_id     Which page was the clicked button on.
+	 * @param element_id  Which button element was clicked.
+	 */
+	ScriptEventStoryPageButtonClick(CompanyID company_id, StoryPageID page_id, StoryPageElementID element_id) :
+		ScriptEvent(ET_STORYPAGE_BUTTON_CLICK),
+		company_id((ScriptCompany::CompanyID)company_id),
+		page_id(page_id),
+		element_id(element_id)
+	{ }
+
+	/**
+	 * Convert an ScriptEvent to the real instance.
+	 * @param instance The instance to convert.
+	 * @return The converted instance.
+	 */
+	static ScriptEventStoryPageButtonClick *Convert(ScriptEvent *instance) { return (ScriptEventStoryPageButtonClick *)instance; }
+
+	/** Get the CompanyID of the player that selected a tile. */
+	ScriptCompany::CompanyID GetCompanyID() { return this->company_id; }
+
+	/** Get the StoryPageID of the storybook page the clicked button is located on. */
+	StoryPageID GetStoryPageID() { return this->page_id; }
+
+	/** Get the StoryPageElementID of the button element that was clicked. */
+	StoryPageElementID GetElementID() { return this->element_id; }
+
+private:
+	ScriptCompany::CompanyID company_id;
+	StoryPageID page_id;
+	StoryPageElementID element_id;
+};
+
+/**
+ * Event StoryPageTileSelect, indicating a player clicked a tile selection button on a storybook page, and selected a tile.
+ * @api game
+ */
+class ScriptEventStoryPageTileSelect : public ScriptEvent {
+public:
+	/**
+	 * @param company_id  Which company triggered the event.
+	 * @param page_id     Which page is the used selection button on.
+	 * @param element_id  Which button element was used to select the tile.
+	 * @param tile_index  Which tile was selected by the player.
+	 */
+	ScriptEventStoryPageTileSelect(CompanyID company_id, StoryPageID page_id, StoryPageElementID element_id, TileIndex tile_index) :
+		ScriptEvent(ET_STORYPAGE_TILE_SELECT),
+		company_id((ScriptCompany::CompanyID)company_id),
+		page_id(page_id),
+		element_id(element_id),
+		tile_index(tile_index)
+	{ }
+
+	/**
+	 * Convert an ScriptEvent to the real instance.
+	 * @param instance The instance to convert.
+	 * @return The converted instance.
+	 */
+	static ScriptEventStoryPageTileSelect *Convert(ScriptEvent *instance) { return (ScriptEventStoryPageTileSelect *)instance; }
+
+	/** Get the CompanyID of the player that selected a tile. */
+	ScriptCompany::CompanyID GetCompanyID() { return this->company_id; }
+
+	/** Get the StoryPageID of the storybook page the used selection button is located on. */
+	StoryPageID GetStoryPageID() { return this->page_id; }
+
+	/** Get the StoryPageElementID of the selection button used to select the tile. */
+	StoryPageElementID GetElementID() { return this->element_id; }
+
+	/** Get the TileIndex of the tile the player selected */
+	TileIndex GetTile() { return this->tile_index; }
+
+private:
+	ScriptCompany::CompanyID company_id;
+	StoryPageID page_id;
+	StoryPageElementID element_id;
+	TileIndex tile_index;
+};
+
+/**
+ * Event StoryPageTileSelect, indicating a player clicked a tile selection button on a storybook page, and selected a tile.
+ * @api game
+ */
+class ScriptEventStoryPageVehicleSelect : public ScriptEvent {
+public:
+	/**
+	 * @param company_id  Which company triggered the event.
+	 * @param page_id     Which page is the used selection button on.
+	 * @param element_id  Which button element was used to select the tile.
+	 * @param vehicle_id  Which vehicle was selected by the player.
+	 */
+	ScriptEventStoryPageVehicleSelect(CompanyID company_id, StoryPageID page_id, StoryPageElementID element_id, VehicleID vehicle_id) :
+		ScriptEvent(ET_STORYPAGE_VEHICLE_SELECT),
+		company_id((ScriptCompany::CompanyID)company_id),
+		page_id(page_id),
+		element_id(element_id),
+		vehicle_id(vehicle_id)
+	{ }
+
+	/**
+	 * Convert an ScriptEvent to the real instance.
+	 * @param instance The instance to convert.
+	 * @return The converted instance.
+	 */
+	static ScriptEventStoryPageVehicleSelect *Convert(ScriptEvent *instance) { return (ScriptEventStoryPageVehicleSelect *)instance; }
+
+	/** Get the CompanyID of the player that selected a tile. */
+	ScriptCompany::CompanyID GetCompanyID() { return this->company_id; }
+
+	/** Get the StoryPageID of the storybook page the used selection button is located on. */
+	StoryPageID GetStoryPageID() { return this->page_id; }
+
+	/** Get the StoryPageElementID of the selection button used to select the vehicle. */
+	StoryPageElementID GetElementID() { return this->element_id; }
+
+	/** Get the VehicleID of the vehicle the player selected */
+	VehicleID GetVehicleID() { return this->vehicle_id; }
+
+private:
+	ScriptCompany::CompanyID company_id;
+	StoryPageID page_id;
+	StoryPageElementID element_id;
+	VehicleID vehicle_id;
+};
+
 #endif /* SCRIPT_EVENT_TYPES_HPP */
