@@ -40,7 +40,7 @@ public:
 	/**
 	 * Types of rail known to the game.
 	 */
-	enum RailType {
+	enum RailType : byte {
 		/* Note: these values represent part of the in-game static values */
 		RAILTYPE_INVALID  = ::INVALID_RAILTYPE, ///< Invalid RailType.
 	};
