@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -18,8 +16,7 @@
 
 ScriptSignList::ScriptSignList()
 {
-	Sign *s;
-	FOR_ALL_SIGNS(s) {
+	for (const Sign *s : Sign::Iterate()) {
 		if (ScriptSign::IsValidSign(s->index)) this->AddItem(s->index);
 	}
 }
