@@ -14,7 +14,7 @@
 
 class SoundDriver_Cocoa : public SoundDriver {
 public:
-	const char *Start(const char * const *param) override;
+	const char *Start(const StringList &param) override;
 
 	void Stop() override;
 	const char *GetName() const override { return "cocoa"; }
