@@ -18,6 +18,17 @@ public:
 	/** Called once every tick */
 	virtual void MainLoop() {}
 
+	/**
+	 * Whether the driver has an output from which the user can hear sound.
+	 * Or in other words, whether we should warn the user if no soundset is
+	 * loaded and that loading one would fix the sound problems.
+	 * @return True for all drivers except null.
+	 */
+	virtual bool HasOutput() const
+	{
+		return true;
+	}
+
 	/**
 	 * Get the currently active instance of the sound driver.
 	 */
@@ -26,6 +37,6 @@ public:
 	}
 };
 
-extern char *_ini_sounddriver;
+extern std::string _ini_sounddriver;
 
 #endif /* SOUND_SOUND_DRIVER_HPP */
