@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -13,6 +11,7 @@
 #define SPRITECACHE_H
 
 #include "gfx_type.h"
+#include "spriteloader/spriteloader.hpp"
 
 /** Data structure describing a sprite. */
 struct Sprite {
@@ -27,12 +26,14 @@ extern uint _sprite_cache_size;
 
 typedef void *AllocatorProc(size_t size);
 
-void *GetRawSprite(SpriteID sprite, SpriteType type, AllocatorProc *allocator = NULL);
+void *SimpleSpriteAlloc(size_t size);
+void *GetRawSprite(SpriteID sprite, SpriteType type, AllocatorProc *allocator = nullptr, SpriteEncoder *encoder = nullptr);
 bool SpriteExists(SpriteID sprite);
 
 SpriteType GetSpriteType(SpriteID sprite);
-uint GetOriginFileSlot(SpriteID sprite);
-uint GetSpriteCountForSlot(uint file_slot, SpriteID begin, SpriteID end);
+SpriteFile *GetOriginFile(SpriteID sprite);
+uint32 GetSpriteLocalID(SpriteID sprite);
+uint GetSpriteCountForFile(const std::string &filename, SpriteID begin, SpriteID end);
 uint GetMaxSpriteID();
 
 
@@ -52,10 +53,12 @@ void GfxInitSpriteMem();
 void GfxClearSpriteCache();
 void IncreaseSpriteLRU();
 
-void ReadGRFSpriteOffsets(byte container_version);
+SpriteFile &OpenCachedSpriteFile(const std::string &filename, Subdirectory subdir, bool palette_remap);
+
+void ReadGRFSpriteOffsets(SpriteFile &file);
 size_t GetGRFSpriteOffset(uint32 id);
-bool LoadNextSprite(int load_index, byte file_index, uint file_sprite_id, byte container_version);
-bool SkipSpriteData(byte type, uint16 num);
+bool LoadNextSprite(int load_index, SpriteFile &file, uint file_sprite_id);
+bool SkipSpriteData(SpriteFile &file, byte type, uint16 num);
 void DupSprite(SpriteID old_spr, SpriteID new_spr);
 
 #endif /* SPRITECACHE_H */
