@@ -9,8 +9,8 @@
 
 #include "../stdafx.h"
 #include "../gfx_func.h"
-#include "../fileio_func.h"
 #include "../debug.h"
+#include "../settings_type.h"
 #include "../strings_func.h"
 #include "table/strings.h"
 #include "../error.h"
@@ -23,15 +23,6 @@
 
 extern const byte _palmap_w2d[];
 
-/** The different colour components a sprite can have. */
-enum SpriteColourComponent {
-	SCC_RGB   = 1 << 0, ///< Sprite has RGB.
-	SCC_ALPHA = 1 << 1, ///< Sprite has alpha.
-	SCC_PAL   = 1 << 2, ///< Sprite has palette data.
-	SCC_MASK  = SCC_RGB | SCC_ALPHA | SCC_PAL, ///< Mask of valid colour bits.
-};
-DECLARE_ENUM_AS_BIT_SET(SpriteColourComponent)
-
 /**
  * We found a corrupted sprite. This means that the sprite itself
  * contains invalid data or is too small for the given dimensions.
@@ -40,14 +31,14 @@ DECLARE_ENUM_AS_BIT_SET(SpriteColourComponent)
  * @param line the line where the error occurs.
  * @return always false (to tell loading the sprite failed)
  */
-static bool WarnCorruptSprite(uint8 file_slot, size_t file_pos, int line)
+static bool WarnCorruptSprite(const SpriteFile &file, size_t file_pos, int line)
 {
 	static byte warning_level = 0;
 	if (warning_level == 0) {
-		SetDParamStr(0, FioGetFilename(file_slot));
+		SetDParamStr(0, file.GetSimplifiedFilename());
 		ShowErrorMessage(STR_NEWGRF_ERROR_CORRUPT_SPRITE, INVALID_STRING_ID, WL_ERROR);
 	}
-	DEBUG(sprite, warning_level, "[%i] Loading corrupted sprite from %s at position %i", line, FioGetFilename(file_slot), (int)file_pos);
+	Debug(sprite, warning_level, "[{}] Loading corrupted sprite from {} at position {}", line, file.GetSimplifiedFilename(), file_pos);
 	warning_level = 6;
 	return false;
 }
@@ -55,7 +46,7 @@ static bool WarnCorruptSprite(uint8 file_slot, size_t file_pos, int line)
 /**
  * Decode the image data of a single sprite.
  * @param[in,out] sprite Filled with the sprite image data.
- * @param file_slot File slot.
+ * @param file The file with the sprite data.
  * @param file_pos File position.
  * @param sprite_type Type of the sprite we're decoding.
  * @param num Size of the decompressed sprite.
@@ -65,7 +56,7 @@ static bool WarnCorruptSprite(uint8 file_slot, size_t file_pos, int line)
  * @param container_format Container format of the GRF this sprite is in.
  * @return True if the sprite was successfully loaded.
  */
-bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t file_pos, SpriteType sprite_type, int64 num, byte type, ZoomLevel zoom_lvl, byte colour_fmt, byte container_format)
+bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, SpriteFile &file, size_t file_pos, SpriteType sprite_type, int64 num, byte type, ZoomLevel zoom_lvl, byte colour_fmt, byte container_format)
 {
 	std::unique_ptr<byte[]> dest_orig(new byte[num]);
 	byte *dest = dest_orig.get();
@@ -73,24 +64,24 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 
 	/* Read the file, which has some kind of compression */
 	while (num > 0) {
-		int8 code = FioReadByte();
+		int8 code = file.ReadByte();
 
 		if (code >= 0) {
 			/* Plain bytes to read */
 			int size = (code == 0) ? 0x80 : code;
 			num -= size;
-			if (num < 0) return WarnCorruptSprite(file_slot, file_pos, __LINE__);
+			if (num < 0) return WarnCorruptSprite(file, file_pos, __LINE__);
 			for (; size > 0; size--) {
-				*dest = FioReadByte();
+				*dest = file.ReadByte();
 				dest++;
 			}
 		} else {
 			/* Copy bytes from earlier in the sprite */
-			const uint data_offset = ((code & 7) << 8) | FioReadByte();
-			if (dest - data_offset < dest_orig.get()) return WarnCorruptSprite(file_slot, file_pos, __LINE__);
+			const uint data_offset = ((code & 7) << 8) | file.ReadByte();
+			if (dest - data_offset < dest_orig.get()) return WarnCorruptSprite(file, file_pos, __LINE__);
 			int size = -(code >> 3);
 			num -= size;
-			if (num < 0) return WarnCorruptSprite(file_slot, file_pos, __LINE__);
+			if (num < 0) return WarnCorruptSprite(file, file_pos, __LINE__);
 			for (; size > 0; size--) {
 				*dest = *(dest - data_offset);
 				dest++;
@@ -98,7 +89,7 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 		}
 	}
 
-	if (num != 0) return WarnCorruptSprite(file_slot, file_pos, __LINE__);
+	if (num != 0) return WarnCorruptSprite(file, file_pos, __LINE__);
 
 	sprite->AllocateData(zoom_lvl, sprite->width * sprite->height);
 
@@ -125,7 +116,7 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 
 			do {
 				if (dest + (container_format >= 2 && sprite->width > 256 ? 4 : 2) > dest_orig.get() + dest_size) {
-					return WarnCorruptSprite(file_slot, file_pos, __LINE__);
+					return WarnCorruptSprite(file, file_pos, __LINE__);
 				}
 
 				SpriteLoader::CommonPixel *data;
@@ -151,7 +142,7 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 				data = &sprite->data[y * sprite->width + skip];
 
 				if (skip + length > sprite->width || dest + length * bpp > dest_orig.get() + dest_size) {
-					return WarnCorruptSprite(file_slot, file_pos, __LINE__);
+					return WarnCorruptSprite(file, file_pos, __LINE__);
 				}
 
 				for (int x = 0; x < length; x++) {
@@ -163,8 +154,8 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 					data->a = (colour_fmt & SCC_ALPHA) ? *dest++ : 0xFF;
 					if (colour_fmt & SCC_PAL) {
 						switch (sprite_type) {
-							case ST_NORMAL: data->m = _palette_remap_grf[file_slot] ? _palmap_w2d[*dest] : *dest; break;
-							case ST_FONT:   data->m = min(*dest, 2u); break;
+							case ST_NORMAL: data->m = file.NeedsPaletteRemap() ? _palmap_w2d[*dest] : *dest; break;
+							case ST_FONT:   data->m = std::min<uint>(*dest, 2u); break;
 							default:        data->m = *dest; break;
 						}
 						/* Magic blue. */
@@ -177,12 +168,12 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 		}
 	} else {
 		if (dest_size < sprite->width * sprite->height * bpp) {
-			return WarnCorruptSprite(file_slot, file_pos, __LINE__);
+			return WarnCorruptSprite(file, file_pos, __LINE__);
 		}
 
 		if (dest_size > sprite->width * sprite->height * bpp) {
 			static byte warning_level = 0;
-			DEBUG(sprite, warning_level, "Ignoring " OTTD_PRINTF64 " unused extra bytes from the sprite from %s at position %i", dest_size - sprite->width * sprite->height * bpp, FioGetFilename(file_slot), (int)file_pos);
+			Debug(sprite, warning_level, "Ignoring {} unused extra bytes from the sprite from {} at position {}", dest_size - sprite->width * sprite->height * bpp, file.GetSimplifiedFilename(), file_pos);
 			warning_level = 6;
 		}
 
@@ -199,8 +190,8 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 			sprite->data[i].a = (colour_fmt & SCC_ALPHA) ? *pixel++ : 0xFF;
 			if (colour_fmt & SCC_PAL) {
 				switch (sprite_type) {
-					case ST_NORMAL: sprite->data[i].m = _palette_remap_grf[file_slot] ? _palmap_w2d[*pixel] : *pixel; break;
-					case ST_FONT:   sprite->data[i].m = min(*pixel, 2u); break;
+					case ST_NORMAL: sprite->data[i].m = file.NeedsPaletteRemap() ? _palmap_w2d[*pixel] : *pixel; break;
+					case ST_FONT:   sprite->data[i].m = std::min<uint>(*pixel, 2u); break;
 					default:        sprite->data[i].m = *pixel; break;
 				}
 				/* Magic blue. */
@@ -213,30 +204,31 @@ bool DecodeSingleSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t fi
 	return true;
 }
 
-uint8 LoadSpriteV1(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t file_pos, SpriteType sprite_type, bool load_32bpp)
+uint8 LoadSpriteV1(SpriteLoader::Sprite *sprite, SpriteFile &file, size_t file_pos, SpriteType sprite_type, bool load_32bpp)
 {
 	/* Check the requested colour depth. */
 	if (load_32bpp) return 0;
 
 	/* Open the right file and go to the correct position */
-	FioSeekToFile(file_slot, file_pos);
+	file.SeekTo(file_pos, SEEK_SET);
 
 	/* Read the size and type */
-	int num = FioReadWord();
-	byte type = FioReadByte();
+	int num = file.ReadWord();
+	byte type = file.ReadByte();
 
 	/* Type 0xFF indicates either a colourmap or some other non-sprite info; we do not handle them here */
 	if (type == 0xFF) return 0;
 
 	ZoomLevel zoom_lvl = (sprite_type != ST_MAPGEN) ? ZOOM_LVL_OUT_4X : ZOOM_LVL_NORMAL;
 
-	sprite[zoom_lvl].height = FioReadByte();
-	sprite[zoom_lvl].width  = FioReadWord();
-	sprite[zoom_lvl].x_offs = FioReadWord();
-	sprite[zoom_lvl].y_offs = FioReadWord();
+	sprite[zoom_lvl].height = file.ReadByte();
+	sprite[zoom_lvl].width  = file.ReadWord();
+	sprite[zoom_lvl].x_offs = file.ReadWord();
+	sprite[zoom_lvl].y_offs = file.ReadWord();
+	sprite[zoom_lvl].colours = SCC_PAL;
 
 	if (sprite[zoom_lvl].width > INT16_MAX) {
-		WarnCorruptSprite(file_slot, file_pos, __LINE__);
+		WarnCorruptSprite(file, file_pos, __LINE__);
 		return 0;
 	}
 
@@ -244,12 +236,12 @@ uint8 LoadSpriteV1(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t file_po
 	 * In case it is uncompressed, the size is 'num' - 8 (header-size). */
 	num = (type & 0x02) ? sprite[zoom_lvl].width * sprite[zoom_lvl].height : num - 8;
 
-	if (DecodeSingleSprite(&sprite[zoom_lvl], file_slot, file_pos, sprite_type, num, type, zoom_lvl, SCC_PAL, 1)) return 1 << zoom_lvl;
+	if (DecodeSingleSprite(&sprite[zoom_lvl], file, file_pos, sprite_type, num, type, zoom_lvl, SCC_PAL, 1)) return 1 << zoom_lvl;
 
 	return 0;
 }
 
-uint8 LoadSpriteV2(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t file_pos, SpriteType sprite_type, bool load_32bpp)
+uint8 LoadSpriteV2(SpriteLoader::Sprite *sprite, SpriteFile &file, size_t file_pos, SpriteType sprite_type, bool load_32bpp)
 {
 	static const ZoomLevel zoom_lvl_map[6] = {ZOOM_LVL_OUT_4X, ZOOM_LVL_NORMAL, ZOOM_LVL_OUT_2X, ZOOM_LVL_OUT_8X, ZOOM_LVL_OUT_16X, ZOOM_LVL_OUT_32X};
 
@@ -257,39 +249,48 @@ uint8 LoadSpriteV2(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t file_po
 	if (file_pos == SIZE_MAX) return 0;
 
 	/* Open the right file and go to the correct position */
-	FioSeekToFile(file_slot, file_pos);
+	file.SeekTo(file_pos, SEEK_SET);
 
-	uint32 id = FioReadDword();
+	uint32 id = file.ReadDword();
 
 	uint8 loaded_sprites = 0;
 	do {
-		int64 num = FioReadDword();
-		size_t start_pos = FioGetPos();
-		byte type = FioReadByte();
+		int64 num = file.ReadDword();
+		size_t start_pos = file.GetPos();
+		byte type = file.ReadByte();
 
 		/* Type 0xFF indicates either a colourmap or some other non-sprite info; we do not handle them here. */
 		if (type == 0xFF) return 0;
 
 		byte colour = type & SCC_MASK;
-		byte zoom = FioReadByte();
+		byte zoom = file.ReadByte();
 
-		if (colour != 0 && (load_32bpp ? colour != SCC_PAL : colour == SCC_PAL) && (sprite_type != ST_MAPGEN ? zoom < lengthof(zoom_lvl_map) : zoom == 0)) {
+		bool is_wanted_colour_depth = (colour != 0 && (load_32bpp ? colour != SCC_PAL : colour == SCC_PAL));
+		bool is_wanted_zoom_lvl;
+
+		if (sprite_type != ST_MAPGEN) {
+			is_wanted_zoom_lvl = (zoom < lengthof(zoom_lvl_map) && zoom_lvl_map[zoom] >= _settings_client.gui.sprite_zoom_min);
+		} else {
+			is_wanted_zoom_lvl = (zoom == 0);
+		}
+
+		if (is_wanted_colour_depth && is_wanted_zoom_lvl) {
 			ZoomLevel zoom_lvl = (sprite_type != ST_MAPGEN) ? zoom_lvl_map[zoom] : ZOOM_LVL_NORMAL;
 
 			if (HasBit(loaded_sprites, zoom_lvl)) {
 				/* We already have this zoom level, skip sprite. */
-				DEBUG(sprite, 1, "Ignoring duplicate zoom level sprite %u from %s", id, FioGetFilename(file_slot));
-				FioSkipBytes(num - 2);
+				Debug(sprite, 1, "Ignoring duplicate zoom level sprite {} from {}", id, file.GetSimplifiedFilename());
+				file.SkipBytes(num - 2);
 				continue;
 			}
 
-			sprite[zoom_lvl].height = FioReadWord();
-			sprite[zoom_lvl].width  = FioReadWord();
-			sprite[zoom_lvl].x_offs = FioReadWord();
-			sprite[zoom_lvl].y_offs = FioReadWord();
+			sprite[zoom_lvl].height = file.ReadWord();
+			sprite[zoom_lvl].width  = file.ReadWord();
+			sprite[zoom_lvl].x_offs = file.ReadWord();
+			sprite[zoom_lvl].y_offs = file.ReadWord();
 
 			if (sprite[zoom_lvl].width > INT16_MAX || sprite[zoom_lvl].height > INT16_MAX) {
-				WarnCorruptSprite(file_slot, file_pos, __LINE__);
+				WarnCorruptSprite(file, file_pos, __LINE__);
 				return 0;
 			}
 
@@ -302,32 +303,34 @@ uint8 LoadSpriteV2(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t file_po
 			if (colour & SCC_ALPHA) bpp++;    // Has alpha data.
 			if (colour & SCC_PAL)   bpp++;    // Has palette data.
 
+			sprite[zoom_lvl].colours = (SpriteColourComponent)colour;
+
 			/* For chunked encoding we store the decompressed size in the file,
 			 * otherwise we can calculate it from the image dimensions. */
-			uint decomp_size = (type & 0x08) ? FioReadDword() : sprite[zoom_lvl].width * sprite[zoom_lvl].height * bpp;
+			uint decomp_size = (type & 0x08) ? file.ReadDword() : sprite[zoom_lvl].width * sprite[zoom_lvl].height * bpp;
 
-			bool valid = DecodeSingleSprite(&sprite[zoom_lvl], file_slot, file_pos, sprite_type, decomp_size, type, zoom_lvl, colour, 2);
-			if (FioGetPos() != start_pos + num) {
-				WarnCorruptSprite(file_slot, file_pos, __LINE__);
+			bool valid = DecodeSingleSprite(&sprite[zoom_lvl], file, file_pos, sprite_type, decomp_size, type, zoom_lvl, colour, 2);
+			if (file.GetPos() != start_pos + num) {
+				WarnCorruptSprite(file, file_pos, __LINE__);
 				return 0;
 			}
 
 			if (valid) SetBit(loaded_sprites, zoom_lvl);
 		} else {
 			/* Not the wanted zoom level or colour depth, continue searching. */
-			FioSkipBytes(num - 2);
+			file.SkipBytes(num - 2);
 		}
 
-	} while (FioReadDword() == id);
+	} while (file.ReadDword() == id);
 
 	return loaded_sprites;
 }
 
-uint8 SpriteLoaderGrf::LoadSprite(SpriteLoader::Sprite *sprite, uint8 file_slot, size_t file_pos, SpriteType sprite_type, bool load_32bpp)
+uint8 SpriteLoaderGrf::LoadSprite(SpriteLoader::Sprite *sprite, SpriteFile &file, size_t file_pos, SpriteType sprite_type, bool load_32bpp)
 {
 	if (this->container_ver >= 2) {
-		return LoadSpriteV2(sprite, file_slot, file_pos, sprite_type, load_32bpp);
+		return LoadSpriteV2(sprite, file, file_pos, sprite_type, load_32bpp);
 	} else {
-		return LoadSpriteV1(sprite, file_slot, file_pos, sprite_type, load_32bpp);
+		return LoadSpriteV1(sprite, file, file_pos, sprite_type, load_32bpp);
 	}
 }
