@@ -97,7 +97,7 @@ bool VerifyTownName(uint32 r, const TownNameParams *par, TownNames *town_names)
 		for (const Town *t : Town::Iterate()) {
 			/* We can't just compare the numbers since
 			 * several numbers may map to a single name. */
-			const char *buf = t->name;
+			const char *buf = t->name.empty() ? nullptr : t->name.c_str();
 			if (buf == nullptr) {
 				GetTownName(buf2, t, lastof(buf2));
 				buf = buf2;
@@ -505,7 +505,7 @@ static char *MakeFinnishTownName(char *buf, const char *last, uint32 seed)
 				strstr(orig, "A") != nullptr || strstr(orig, "O") != nullptr || strstr(orig, "U")  != nullptr) {
 			buf = strecpy(buf, "la", last);
 		} else {
-			buf = strecpy(buf, "l\xC3\xA4", last);
+			buf = strecpy(buf, u8"l\u00e4", last);
 		}
 		return buf;
 	}
@@ -600,7 +600,7 @@ static char *MakeCzechTownName(char *buf, const char *last, uint32 seed)
 		return strecpy(buf, _name_czech_real[SeedModChance(4, lengthof(_name_czech_real), seed)], last);
 	}
 
-	const char *orig = buf;
+	[[maybe_unused]] const char *orig = buf;
 
 	/* Probability of prefixes/suffixes
 	 * 0..11 prefix, 12..13 prefix+suffix, 14..17 suffix, 18..31 nothing */
