@@ -67,9 +67,9 @@ static int HighlightDragPosition(int px, int max_width, VehicleID selection, boo
 		dragged_width += t->GetDisplayImageWidth(nullptr);
 	}
 
-	int drag_hlight_left = rtl ? max(px - dragged_width + 1, 0) : px;
-	int drag_hlight_right = rtl ? px : min(px + dragged_width, max_width) - 1;
-	int drag_hlight_width = max(drag_hlight_right - drag_hlight_left + 1, 0);
+	int drag_hlight_left = rtl ? std::max(px - dragged_width + 1, 0) : px;
+	int drag_hlight_right = rtl ? px : std::min(px + dragged_width, max_width) - 1;
+	int drag_hlight_width = std::max(drag_hlight_right - drag_hlight_left + 1, 0);
 
 	if (drag_hlight_width > 0) {
 		GfxFillRect(drag_hlight_left + WD_FRAMERECT_LEFT, WD_FRAMERECT_TOP + 1,
@@ -337,7 +337,7 @@ int GetTrainDetailsWndVScroll(VehicleID veh_id, TrainDetailsWindowTabs det_tab)
 	} else {
 		for (const Train *v = Train::Get(veh_id); v != nullptr; v = v->GetNextVehicle()) {
 			GetCargoSummaryOfArticulatedVehicle(v, &_cargo_summary);
-			num += max(1u, (unsigned)_cargo_summary.size());
+			num += std::max(1u, (unsigned)_cargo_summary.size());
 
 			uint length = GetLengthOfArticulatedVehicle(v);
 			if (length > TRAIN_DETAILS_MAX_INDENT) num++;
@@ -364,7 +364,7 @@ void DrawTrainDetails(const Train *v, int left, int right, int y, int vscroll_po
 	y -= WD_MATRIX_TOP;
 
 	int sprite_height = ScaleGUITrad(GetVehicleHeight(VEH_TRAIN));
-	int line_height = max(sprite_height, WD_MATRIX_TOP + FONT_HEIGHT_NORMAL + WD_MATRIX_BOTTOM);
+	int line_height = std::max(sprite_height, WD_MATRIX_TOP + FONT_HEIGHT_NORMAL + WD_MATRIX_BOTTOM);
 	int sprite_y_offset = line_height / 2;
 	int text_y_offset = (line_height - FONT_HEIGHT_NORMAL) / 2;
 
@@ -405,9 +405,9 @@ void DrawTrainDetails(const Train *v, int left, int right, int y, int vscroll_po
 				dx = 0;
 			}
 
-			uint num_lines = max(1u, (unsigned)_cargo_summary.size());
+			uint num_lines = std::max(1u, (unsigned)_cargo_summary.size());
 			for (uint i = 0; i < num_lines; i++) {
-				int sprite_width = max<int>(dx, ScaleGUITrad(TRAIN_DETAILS_MIN_INDENT)) + 3;
+				int sprite_width = std::max<int>(dx, ScaleGUITrad(TRAIN_DETAILS_MIN_INDENT)) + 3;
 				int data_left  = left + (rtl ? 0 : sprite_width);
 				int data_right = right - (rtl ? sprite_width : 0);
 				if (vscroll_pos <= 0 && vscroll_pos > -vscroll_cap) {
