@@ -25,7 +25,7 @@ uint32 VehicleListIdentifier::Pack() const
 	assert(this->vtype   < (1 <<  2));
 	assert(this->index   < (1 << 20));
 	assert(this->type    < VLT_END);
-	assert_compile(VLT_END <= (1 <<  3));
+	static_assert(VLT_END <= (1 <<  3));
 
 	return c << 28 | this->type << 23 | this->vtype << 26 | this->index;
 }
@@ -53,7 +53,7 @@ bool VehicleListIdentifier::UnpackIfValid(uint32 data)
 /* static */ VehicleListIdentifier VehicleListIdentifier::UnPack(uint32 data)
 {
 	VehicleListIdentifier result;
-	bool ret = result.UnpackIfValid(data);
+	[[maybe_unused]] bool ret = result.UnpackIfValid(data);
 	assert(ret);
 	return result;
 }
@@ -118,9 +118,7 @@ bool GenerateVehicleSortList(VehicleList *list, const VehicleListIdentifier &vli
 		case VL_STATION_LIST:
 			for (const Vehicle *v : Vehicle::Iterate()) {
 				if (v->type == vli.vtype && v->IsPrimaryVehicle()) {
-					const Order *order;
-
-					FOR_VEHICLE_ORDERS(v, order) {
+					for (const Order *order : v->Orders()) {
 						if ((order->IsType(OT_GOTO_STATION) || order->IsType(OT_GOTO_WAYPOINT) || order->IsType(OT_IMPLICIT))
 								&& order->GetDestination() == vli.index) {
 							list->push_back(v);
@@ -165,9 +163,7 @@ bool GenerateVehicleSortList(VehicleList *list, const VehicleListIdentifier &vli
 		case VL_DEPOT_LIST:
 			for (const Vehicle *v : Vehicle::Iterate()) {
 				if (v->type == vli.vtype && v->IsPrimaryVehicle()) {
-					const Order *order;
-
-					FOR_VEHICLE_ORDERS(v, order) {
+					for (const Order *order : v->Orders()) {
 						if (order->IsType(OT_GOTO_DEPOT) && !(order->GetDepotActionType() & ODATFB_NEAREST_DEPOT) && order->GetDestination() == vli.index) {
 							list->push_back(v);
 							break;
