@@ -15,6 +15,13 @@ enum BootstrapBackgroundWidgets {
 	WID_BB_BACKGROUND, ///< Background of the window.
 };
 
+/** Widgets of the #BootstrapErrmsgWindow class. */
+enum BootstrapErrorMessageWidgets {
+	WID_BEM_CAPTION, ///< Caption of the window.
+	WID_BEM_MESSAGE, ///< Error message.
+	WID_BEM_QUIT,    ///< Quit button.
+};
+
 /** Widgets of the #BootstrapContentDownloadStatusWindow class. */
 enum BootstrapAskForDownloadWidgets {
 	WID_BAFD_QUESTION, ///< The question whether to download.
